@@ -18,6 +18,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*! \file
   This file defines the class AutocompletePopup.
@@ -126,6 +127,7 @@ void AutocompletePopup::OnKeyDown(wxKeyEvent &event)
     Destroy();
     break;
   case WXK_UP:
+  case WXK_NUMPAD_UP:
   {
     int selection = GetNextItem(0,wxLIST_NEXT_ALL,
                                 wxLIST_STATE_SELECTED);
@@ -143,6 +145,7 @@ void AutocompletePopup::OnKeyDown(wxKeyEvent &event)
     break;
   }
   case WXK_PAGEUP:
+  case WXK_NUMPAD_PAGEUP:
   {
     int selection = GetNextItem(0,wxLIST_NEXT_ALL,
                                 wxLIST_STATE_SELECTED);
@@ -158,6 +161,7 @@ void AutocompletePopup::OnKeyDown(wxKeyEvent &event)
     break;
   }
   case WXK_PAGEDOWN:
+  case WXK_NUMPAD_PAGEDOWN:
   {
     int selection = GetNextItem(0,wxLIST_NEXT_ALL,
                                 wxLIST_STATE_SELECTED);
@@ -173,6 +177,7 @@ void AutocompletePopup::OnKeyDown(wxKeyEvent &event)
     break;
   }
   case WXK_DOWN:
+  case WXK_NUMPAD_DOWN:
   {
     long selection = GetNextItem(0,wxLIST_NEXT_ALL,
                                  wxLIST_STATE_SELECTED);
@@ -187,6 +192,7 @@ void AutocompletePopup::OnKeyDown(wxKeyEvent &event)
     break;
   }
   case WXK_BACK:
+  case WXK_NUMPAD_DELETE:
   {
     wxString oldString = m_editor->GetSelectionString();
     if (oldString != wxEmptyString)
