@@ -20,6 +20,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*\file
 
@@ -40,7 +41,6 @@
 #include <wx/colordlg.h>
 #include <wx/settings.h>
 #include <wx/filename.h>
-#include "Dirstructure.h"
 #include "../art/config/images.h"
 #include <wx/mstream.h>
 #include <wx/wfstream.h>
@@ -51,7 +51,7 @@
 #define MAX(a, b) ((a)>(b) ? (a) : (b))
 #define MIN(a, b) ((a)>(b) ? (b) : (a))
 
-/*! The enum that chooses the language in the language drop-down menu. 
+/*! The enum that chooses the language in the language drop-down menu.
 
   \attention
   - Should match whatever is put in the m_language.
@@ -108,7 +108,7 @@ int ConfigDialogue::GetImageSize()
   }
 }
 
-wxImage ConfigDialogue::GetImage(wxString name,
+wxBitmap ConfigDialogue::GetImage(wxString name,
                           unsigned char *data_128, size_t len_128,
                           unsigned char *data_192, size_t len_192)
 {
@@ -161,7 +161,7 @@ wxImage ConfigDialogue::GetImage(wxString name,
 
   img.Rescale(targetSize, targetSize, wxIMAGE_QUALITY_HIGH);
 
-  return img;
+  return wxBitmap(img,wxBITMAP_SCREEN_DEPTH);
 }
 
 
@@ -175,9 +175,6 @@ ConfigDialogue::ConfigDialogue(wxWindow *parent, Configuration *cfg)
 #endif
   SetSheetInnerBorder(3);
   SetSheetOuterBorder(3);
-  // Allow the property dialogue sheets to scroll, if they don't fit
-  // on the screen.
-  SetLayoutAdaptationMode(wxDIALOG_ADAPTATION_MODE_ENABLED);
 
   int imgSize = GetImageSize();
   m_imageList = new wxImageList(imgSize, imgSize);
@@ -236,6 +233,11 @@ ConfigDialogue::ConfigDialogue(wxWindow *parent, Configuration *cfg)
   if((notebookTab < 0) || ((unsigned int)notebookTab > m_notebook->GetPageCount()))
      notebookTab = 0;
   m_notebook->SetSelection(notebookTab);
+
+  // Allow the property dialogue sheets to scroll, if they don't fit
+  // on the screen.
+  SetLayoutAdaptationMode(wxDIALOG_ADAPTATION_MODE_ENABLED);
+
   LayoutDialog();
 
   SetProperties();
@@ -293,8 +295,6 @@ void ConfigDialogue::SetProperties()
   m_abortOnError->SetToolTip(
           _("If multiple cells are evaluated in one go: Abort evaluation if wxMaxima detects that maxima has encountered any error."));
   m_openHCaret->SetToolTip("If this checkbox is set a new code cell is opened as soon as maxima requests data. If it isn't set a new code cell is opened in this case as soon as the user starts typing in code.");
-  m_pollStdOut->SetToolTip(
-          _("Once the local network link between maxima and wxMaxima has been established maxima has no reason to send any messages using the system's stdout stream so all this stream transport should be a greeting message; The lisp running maxima will send eventual error messages using the system's stderr stream instead. If this box is checked we will nonetheless watch maxima's stdout stream for messages."));
   m_restartOnReEvaluation->SetToolTip(
           _("Maxima provides no \"forget all\" command that flushes all settings a maxima session could make. wxMaxima therefore normally defaults to starting a fresh maxima process every time the worksheet is to be re-evaluated. As this needs a little bit of time this switch allows to disable this behavior."));
   m_maximaProgram->SetToolTip(_("Enter the path to the Maxima executable."));
@@ -343,10 +343,8 @@ void ConfigDialogue::SetProperties()
   m_autosubscript->SetToolTip(
           _("false=Don't generate subscripts\ntrue=Automatically convert underscores to subscript markers if the would-be subscript is a number or a single letter\nall=_ marks subscripts."));
   m_language->SetToolTip(_("Language used for wxMaxima GUI."));
-#ifdef wxUSE_UNICODE
   m_symbolPaneAdditionalChars->SetToolTip(
           _("Symbols that are entered or copied here will appear in the symbols sidebar so they can be entered into the worksheet easily."));
-#endif
   m_documentclass->SetToolTip(_("The document class LaTeX is instructed to use for our documents."));
   m_fixedFontInTC->SetToolTip(_("Set fixed font in text controls."));
   m_getFont->SetToolTip(_("Font used for display in document."));
@@ -361,20 +359,16 @@ void ConfigDialogue::SetProperties()
 
   m_hideBrackets->SetToolTip(
           _("Hide the brackets that mark the extend of the worksheet cells at the worksheet's right side and that contain the \"hide\" button of the cell if the cells aren't active."));
+  m_indentMaths->SetToolTip(
+          _("Indent maths so all lines are in par with the first line that starts after the label."));
 
-#ifdef __WXMSW__
-  m_wxcd->SetToolTip(_("Automatically change maxima's working directory to the one the current document is in: "
-                       "This is necessary if the document uses File I/O relative to the current directory "
-                       "but will make maxima 5.35 fail to find its own installation path when the current "
-                       "document resides on a different drive than the maxima installation."));
-#endif
   wxConfig *config = (wxConfig *) wxConfig::Get();
   wxString mp, mc, ib, mf;
 
   // The default values for all config items that will be used if there is no saved
   // configuration data for this item.
   bool savePanes = true;
-  bool fixedFontTC = true, usejsmath = true, keepPercent = true, abortOnError = true, pollStdOut = false;
+  bool fixedFontTC = true, usejsmath = true, keepPercent = true;
   bool enterEvaluates = false, saveUntitled = true,
           AnimateLaTeX = true, TeXExponentsAfterSubscript = false,
           usePartialForDiff = false,
@@ -386,17 +380,13 @@ void ConfigDialogue::SetProperties()
   int labelWidth = 4;
   int undoLimit = 0;
   int recentItems = 10;
-  int autosubscript = 1;
   int bitmapScale = 3;
   bool incrementalSearch = true;
   int defaultFramerate = 2;
   wxString texPreamble = wxEmptyString;
   wxString documentclass = wxT("article");
-#ifdef wxUSE_UNICODE
   wxString symbolPaneAdditionalChars = wxT("üØ");
   m_symbolPaneAdditionalChars->SetValue(symbolPaneAdditionalChars);
-#endif
-  int autoSaveInterval = 3;
 
 #if defined (__WXMAC__)
   bool usepngCairo = false;
@@ -426,11 +416,9 @@ void ConfigDialogue::SetProperties()
   config->Read(wxT("exportContainsWXMX"), &exportContainsWXMX);
   config->Read(wxT("HTMLequationFormat"), &exportWithMathJAX);
   config->Read(wxT("pos-restore"), &rs);
-  config->Read(wxT("autosubscript"), &autosubscript);
   config->Read(wxT("language"), &lang);
   config->Read(wxT("documentclass"), &documentclass);
   config->Read(wxT("texPreamble"), &texPreamble);
-  config->Read(wxT("autoSaveInterval"), &autoSaveInterval);
   config->Read(wxT("fixedFontTC"), &fixedFontTC);
   config->Read(wxT("panelSize"), &panelSize);
   config->Read(wxT("enterEvaluates"), &enterEvaluates);
@@ -443,8 +431,6 @@ void ConfigDialogue::SetProperties()
   config->Read(wxT("incrementalSearch"), &incrementalSearch);
   config->Read(wxT("usejsmath"), &usejsmath);
   config->Read(wxT("keepPercent"), &keepPercent);
-  config->Read(wxT("abortOnError"), &abortOnError);
-  config->Read(wxT("pollStdOut"), &pollStdOut);
   unsigned int i = 0;
   for (i = 0; i < LANGUAGE_NUMBER; i++)
     if (langs[i] == lang)
@@ -454,14 +440,12 @@ void ConfigDialogue::SetProperties()
   else
     m_language->SetSelection(0);
 
-#ifdef wxUSE_UNICODE
   config->Read(wxT("symbolPaneAdditionalChars"), &symbolPaneAdditionalChars);
-#endif
 
   m_documentclass->SetValue(documentclass);
   m_mathJaxURL->SetValue(configuration->MathJaXURL());
   m_texPreamble->SetValue(texPreamble);
-  m_autoSaveInterval->SetValue(autoSaveInterval);
+  m_autoSaveInterval->SetValue(configuration->AutoSaveInterval() /  60 / 1000);
 
   m_maximaProgram->SetValue(configuration->MaximaLocation());
   wxCommandEvent dummy;
@@ -488,7 +472,7 @@ void ConfigDialogue::SetProperties()
   m_exportWithMathJAX->SetSelection(exportWithMathJAX);
   m_matchParens->SetValue(configuration->GetMatchParens());
   m_showLength->SetSelection(configuration->ShowLength());
-  m_autosubscript->SetSelection(autosubscript);
+  m_autosubscript->SetSelection(configuration->GetAutosubscript_Num());
   m_changeAsterisk->SetValue(configuration->GetChangeAsterisk());
   m_enterEvaluates->SetValue(enterEvaluates);
   m_saveUntitled->SetValue(saveUntitled);
@@ -497,6 +481,7 @@ void ConfigDialogue::SetProperties()
   m_autoIndent->SetValue(configuration->GetAutoIndent());
   m_cursorJump->SetValue(cursorJump);
   m_hideBrackets->SetValue(configuration->HideBrackets());
+  m_indentMaths->SetValue(configuration->IndentMaths());
   int val = 0;
   if (configuration->GetAutoWrap()) val = 1;
 //  if(configuration->GetAutoWrapCode()) val = 2;
@@ -512,17 +497,17 @@ void ConfigDialogue::SetProperties()
   m_fixedFontInTC->SetValue(fixedFontTC);
   m_useJSMath->SetValue(usejsmath);
   m_keepPercentWithSpecials->SetValue(keepPercent);
-  m_abortOnError->SetValue(abortOnError);
-  m_pollStdOut->SetValue(pollStdOut);
+  m_abortOnError->SetValue(configuration->GetAbortOnError());
   m_restartOnReEvaluation->SetValue(configuration->RestartOnReEvaluation());
   m_defaultFramerate->SetValue(defaultFramerate);
   m_defaultPlotWidth->SetValue(defaultPlotWidth);
   m_defaultPlotHeight->SetValue(defaultPlotHeight);
   m_displayedDigits->SetValue(configuration->GetDisplayedDigits());
-#ifdef wxUSE_UNICODE
   m_symbolPaneAdditionalChars->SetValue(symbolPaneAdditionalChars);
-#endif
-  m_getStyleFont->Enable(false);
+  if (m_styleFor->GetSelection() >= 14 && m_styleFor->GetSelection() <= 18)
+    m_getStyleFont->Enable(true);
+  else
+    m_getStyleFont->Enable(false);
 
   if (!wxFontEnumerator::IsValidFacename(CMEX10) ||
       !wxFontEnumerator::IsValidFacename(CMSY10) ||
@@ -538,7 +523,7 @@ wxPanel *ConfigDialogue::CreateWorksheetPanel()
   wxPanel *panel = new wxPanel(m_notebook, -1);
 
   wxFlexGridSizer *grid_sizer = new wxFlexGridSizer(10, 2, 5, 5);
-  wxFlexGridSizer *vsizer = new wxFlexGridSizer(19, 1, 5, 5);
+  wxFlexGridSizer *vsizer = new wxFlexGridSizer(29, 1, 5, 5);
 
   wxStaticText *pw = new wxStaticText(panel, -1, _("Default plot size for new maxima sessions:"));
   wxBoxSizer *PlotWidthHbox = new wxBoxSizer(wxHORIZONTAL);
@@ -582,7 +567,7 @@ wxPanel *ConfigDialogue::CreateWorksheetPanel()
   wxStaticText *as = new wxStaticText(panel, -1, _("Underscore converts to subscripts:"));
   grid_sizer->Add(as, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   wxArrayString autosubscripts;
-  autosubscripts.Add(_("No"));
+  autosubscripts.Add(_("Never"));
   autosubscripts.Add(_("Integers and single letters"));
   autosubscripts.Add(_("All variable names"));
   m_autosubscript = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, autosubscripts);
@@ -607,10 +592,12 @@ wxPanel *ConfigDialogue::CreateWorksheetPanel()
 
   vsizer->Add(grid_sizer, 1, wxEXPAND, 5);
 
-
   m_hideBrackets = new wxCheckBox(panel, -1, _("Intelligently hide cell brackets"));
   vsizer->Add(m_hideBrackets, 0, wxALL, 5);
 
+  m_indentMaths = new wxCheckBox(panel, -1, _("Indent equations by the label width"));
+  vsizer->Add(m_indentMaths, 0, wxALL, 5);
+
   m_enterEvaluates = new wxCheckBox(panel, -1, _("Enter evaluates cells"));
   vsizer->Add(m_enterEvaluates, 0, wxALL, 5);
 
@@ -655,20 +642,19 @@ wxPanel *ConfigDialogue::CreateStartupPanel()
   wxBoxSizer *vsizer_maximaStartup = new wxBoxSizer(wxVERTICAL);
   wxBoxSizer *vsizer_wxMaximaStartup = new wxBoxSizer(wxVERTICAL);
 
-  Dirstructure dirstruct;
-  
-  m_startupFileName = dirstruct.MaximaUserFilesDir();
+
+  m_startupFileName = m_configuration->m_dirStructure.UserConfDir();
   m_wxStartupFileName += m_startupFileName + wxT("wxmaxima-init.mac");
   m_startupFileName += wxT("maxima-init.mac");
 
   wxStaticText *wxStartupText =
     new wxStaticText(panel_wxMaximaStartup, wxID_ANY,
                      _("Maxima commands to be executed every time wxMaxima starts Maxima: "));
-  wxStartupText->SetToolTip(_("The part of the output of these commands that isn' declared as "
-                              "\"math\" might be suppressed by wxMaxima; As always maxima "
+  wxStartupText->SetToolTip(_("The part of the output of these commands that isn't declared as "
+                              "\"math\" might be suppressed by wxMaxima. As always maxima "
                               "commands are required to end in a \";\" or a \"$\""));
   vsizer_wxMaximaStartup->Add(wxStartupText, wxSizerFlags().Border(wxALL,5));
-  
+
   // Read the contents of wxMaxima's startup file
   wxString contents;
   if(wxFileExists(m_wxStartupFileName))
@@ -692,12 +678,9 @@ wxPanel *ConfigDialogue::CreateStartupPanel()
   wxStaticText *wxStartupFileLocation = new wxStaticText(panel_wxMaximaStartup, wxID_ANY,
                                                          _("wxMaxima startup file location: ") +
                                                          m_wxStartupFileName);
-  wxStartupFileLocation->SetToolTip(_("If wxMaxima's guess where maxima expects the startup file "
-                                      "to be is wrong the right directory for the startup file is " 
-                                      "the one maxima prints out in response to the command\n\n"
-                                      "   maxima-user_dir;\n\n"
-                                      "The file won't be read by maxima if wxMaxima isn't in use. "
-                                      "To add startup commands that are executed in this case, "
+  wxStartupFileLocation->SetToolTip(_("This file won't be read by maxima if maxima is used "
+                                      "without wxMaxima. In order to add startup commands "
+                                      "that are executed in this case, "
                                       "too, please add them to maxima-init.mac, instead."));
   vsizer_wxMaximaStartup->Add(wxStartupFileLocation, wxSizerFlags().Border(wxALL,5));
 
@@ -707,8 +690,8 @@ wxPanel *ConfigDialogue::CreateStartupPanel()
   wxStaticText *startupText =
     new wxStaticText(panel_maximaStartup, wxID_ANY,
                      _("Maxima commands to be executed at every start of Maxima: "));
-  startupText->SetToolTip(_("The part of the output of these commands that isn' declared as "
-                              "\"math\" might be suppressed by wxMaxima; As always maxima "
+  startupText->SetToolTip(_("The part of the output of these commands that isn't declared as "
+                              "\"math\" might be suppressed by wxMaxima. As always maxima "
                               "commands are required to end in a \";\" or a \"$\""));
   vsizer_maximaStartup->Add(startupText, wxSizerFlags().Border(wxALL,5));
   // Read maxima's startup file's contents
@@ -735,13 +718,7 @@ wxPanel *ConfigDialogue::CreateStartupPanel()
   wxStaticText *startupFileLocation = new wxStaticText(panel_maximaStartup, wxID_ANY,
                                                        _("Maxima startup file location: ") +
                                                        m_startupFileName);
-  startupFileLocation->SetToolTip(_("If wxMaxima's guess where maxima expects the startup file "
-                                    "is wrong the right directory for the startup file is the "
-                                    "one maxima prints out in response to the command\n\n"
-                                    "   maxima-user_dir;\n\n"
-                                    "The file won't be read by maxima if wxMaxima isn't in use. "
-                                    "To add startup commands that are executed in this case, "
-                                    "too, please add them to maxima-init.mac, instead."));
+  startupFileLocation->SetToolTip(_("Commands that are executed at every start of maxima."));
   vsizer_maximaStartup->Add(startupFileLocation, wxSizerFlags().Border(wxALL,5));
   panel_maximaStartup->SetSizerAndFit(vsizer_maximaStartup);
   vsizer->Add(panel_maximaStartup,wxSizerFlags().Expand().Border(wxTOP, 5));
@@ -794,7 +771,7 @@ wxPanel *ConfigDialogue::CreateExportPanel()
   m_printScale->SetIncrement(.1);
   grid_sizer->Add(ps, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   grid_sizer->Add(m_printScale, 0, wxALL, 5);
-  
+
   m_AnimateLaTeX = new wxCheckBox(panel, -1,
                                   _("Export animations to TeX (Images only move if the PDF viewer supports this)"));
   vsizer->Add(m_AnimateLaTeX, 0, wxALL, 5);
@@ -863,14 +840,12 @@ wxPanel *ConfigDialogue::CreateOptionsPanel()
   grid_sizer->Add(lang, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   grid_sizer->Add(m_language, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
 
-#ifdef wxUSE_UNICODE
   wxStaticText *additionalSymbols = new wxStaticText(panel, -1, _("Additional symbols for the \"symbols\" sidebar:"));
   m_symbolPaneAdditionalChars = new wxTextCtrl(panel, -1);
   grid_sizer->Add(additionalSymbols, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   grid_sizer->Add(m_symbolPaneAdditionalChars, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
-#endif
 
-  wxStaticText *as = new wxStaticText(panel, -1, _("Autosave interval (minutes, 0 means: off):"));
+  wxStaticText *as = new wxStaticText(panel, -1, _("Autosave on close and every n minutes (0 means: off):"));
   m_autoSaveInterval = new wxSpinCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(230, -1), wxSP_ARROW_KEYS, 0,
                                       30);
   grid_sizer->Add(as, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
@@ -904,7 +879,7 @@ wxPanel *ConfigDialogue::CreateOptionsPanel()
   m_usepngCairo->Connect(wxEVT_CHECKBOX,
                          wxCommandEventHandler(ConfigDialogue::UsepngcairoChanged),
                          NULL, this);
-  
+
   vsizer->Add(m_usepngCairo, 0, wxALL, 5);
 
   m_antialiasLines = new wxCheckBox(panel, -1, _("Antialias lines."));
@@ -921,7 +896,7 @@ wxPanel *ConfigDialogue::CreateOptionsPanel()
   m_notifyIfIdle = new wxCheckBox(panel, -1, _("Warn if an inactive window is idle"));
   vsizer->Add(m_notifyIfIdle, 0, wxALL, 5);
 
-  
+
   vsizer->AddGrowableRow(10);
   panel->SetSizer(vsizer);
   vsizer->Fit(panel);
@@ -985,20 +960,10 @@ wxPanel *ConfigDialogue::CreateMaximaPanel()
   vsizer->Add(m_additionalParameters, 0, wxALL, 0);
 
   vsizer->Add(10, 10);
-#ifdef __WXMSW__
-  bool wxcd = true;
-  wxConfig::Get()->Read(wxT("wxcd"), &wxcd);
-  m_wxcd = new wxCheckBox(panel, -1, _("maxima's pwd is path to document"));
-  m_wxcd-> SetValue(wxcd);
-  vsizer->Add(m_wxcd, 0, wxALL, 5);
-  vsizer->Add(10, 10);
-#endif
 
   m_abortOnError = new wxCheckBox(panel, -1, _("Abort evaluation on error"));
   vsizer->Add(m_abortOnError, 0, wxALL, 5);
 
-  m_pollStdOut = new wxCheckBox(panel, -1, _("Debug: watch maxima's stdout stream"));
-  vsizer->Add(m_pollStdOut, 0, wxALL, 5);
   panel->SetSizerAndFit(vsizer);
 
   m_restartOnReEvaluation = new wxCheckBox(panel, -1, _("Start a new maxima for each re-evaluation"));
@@ -1037,6 +1002,12 @@ wxPanel *ConfigDialogue::CreateClipboardPanel()
   m_copySVG->SetValue(configuration->CopySVG());
   vbox->Add(m_copySVG, 0, wxALL, 5);
 
+  #if wxUSE_ENH_METAFILE
+  m_copyEMF = new wxCheckBox(panel, -1, _("Enhanced meta file (emf)"));
+  m_copyEMF->SetValue(configuration->CopyEMF());
+  vbox->Add(m_copyEMF, 0, wxALL, 5);
+  #endif
+
   panel->SetSizerAndFit(vbox);
 
   return panel;
@@ -1113,6 +1084,7 @@ wxPanel *ConfigDialogue::CreateStylePanel()
   m_styleFor->Connect(wxEVT_LISTBOX,
                          wxCommandEventHandler(ConfigDialogue::OnStyleToEditChanged),
                          NULL, this);
+
   m_getStyleFont = new wxButton(panel, style_font_family, _("Choose font"), wxDefaultPosition, wxSize(150, -1));
   m_styleColor = new ColorPanel(this, panel, color_id, wxDefaultPosition, wxSize(150, 30),
                                 wxSUNKEN_BORDER | wxFULL_REPAINT_ON_RESIZE);
@@ -1193,8 +1165,7 @@ void ConfigDialogue::WriteSettings()
   wxArrayString out;
   wxConfig *config = (wxConfig *) wxConfig::Get();
   Configuration *configuration = m_configuration;
-  config->Write(wxT("abortOnError"), m_abortOnError->GetValue());
-  config->Write(wxT("pollStdOut"), m_pollStdOut->GetValue());
+  configuration->SetAbortOnError(m_abortOnError->GetValue());
   configuration->RestartOnReEvaluation(m_restartOnReEvaluation->GetValue());
   if (
           (configuration->MaximaFound()) ||
@@ -1206,7 +1177,7 @@ void ConfigDialogue::WriteSettings()
   config->Write(wxT("mathFontsize"), m_mathFontSize);
   config->Write(wxT("matchParens"), m_matchParens->GetValue());
   configuration->ShowLength(m_showLength->GetSelection());
-  config->Write(wxT("autosubscript"), m_autosubscript->GetSelection());
+  configuration->SetAutosubscript_Num(m_autosubscript->GetSelection());
   config->Write(wxT("fixedFontTC"), m_fixedFontInTC->GetValue());
   configuration->SetChangeAsterisk(m_changeAsterisk->GetValue());
   config->Write(wxT("enterEvaluates"), m_enterEvaluates->GetValue());
@@ -1216,6 +1187,7 @@ void ConfigDialogue::WriteSettings()
   configuration->SetAutoIndent(m_autoIndent->GetValue());
   config->Write(wxT("cursorJump"), m_cursorJump->GetValue());
   configuration->HideBrackets(m_hideBrackets->GetValue());
+  configuration->IndentMaths(m_indentMaths->GetValue());
   configuration->SetAutoWrap(m_autoWrap->GetSelection());
   config->Write(wxT("labelWidth"), m_labelWidth->GetValue());
   config->Write(wxT("undoLimit"), m_undoLimit->GetValue());
@@ -1227,9 +1199,6 @@ void ConfigDialogue::WriteSettings()
   configuration->NotifyIfIdle(m_notifyIfIdle->GetValue());
   configuration->SetLabelChoice(m_showUserDefinedLabels->GetSelection());
   config->Write(wxT("defaultPort"), m_defaultPort->GetValue());
-#ifdef __WXMSW__
-  config->Write(wxT("wxcd"), m_wxcd->GetValue());
-#endif
   config->Write(wxT("AUI/savePanes"), m_savePanes->GetValue());
   config->Write(wxT("usepngCairo"), m_usepngCairo->GetValue());
   configuration->AntiAliasLines(m_antialiasLines->GetValue());
@@ -1248,7 +1217,7 @@ void ConfigDialogue::WriteSettings()
   configuration->UseUnicodeMaths(m_useUnicodeMaths->GetValue());
   config->Write(wxT("keepPercent"), m_keepPercentWithSpecials->GetValue());
   config->Write(wxT("texPreamble"), m_texPreamble->GetValue());
-  config->Write(wxT("autoSaveInterval"), m_autoSaveInterval->GetValue());
+  configuration->AutoSaveInterval(m_autoSaveInterval->GetValue() * 60 * 1000);
   config->Write(wxT("documentclass"), m_documentclass->GetValue());
   configuration->MathJaXURL(m_mathJaxURL->GetValue());
   if (m_saveSize->GetValue())
@@ -1259,16 +1228,17 @@ void ConfigDialogue::WriteSettings()
   i = m_language->GetSelection();
   if (i > -1 && i < LANGUAGE_NUMBER)
     config->Write(wxT("language"), langs[i]);
-#ifdef wxUSE_UNICODE
   config->Write(wxT("symbolPaneAdditionalChars"), m_symbolPaneAdditionalChars->GetValue());
-#endif
 
   configuration->CopyBitmap(m_copyBitmap->GetValue());
   configuration->CopyMathML(m_copyMathML->GetValue());
   configuration->CopyMathMLHTML(m_copyMathMLHTML->GetValue());
   configuration->CopyRTF(m_copyRTF->GetValue());
   configuration->CopySVG(m_copySVG->GetValue());
-  
+  #if wxUSE_ENH_METAFILE
+  configuration->CopyEMF(m_copyEMF->GetValue());
+  #endif
+
   WriteStyles();
   config->Flush();
 
@@ -1278,6 +1248,7 @@ void ConfigDialogue::WriteSettings()
     {
       wxTextOutputStream text(output);
       text << m_startupCommands->GetValue();
+      text.Flush();
     }
   }
 
@@ -1287,7 +1258,8 @@ void ConfigDialogue::WriteSettings()
     {
       wxTextOutputStream text(output);
       text << m_wxStartupCommands->GetValue();
-    }  
+      text.Flush();
+    }
   }
   config->Write(wxT("ConfigDialogTab"), m_notebook->GetSelection());
 }
@@ -1330,7 +1302,7 @@ void ConfigDialogue::OnMathBrowse(wxCommandEvent&  WXUNUSED(event))
     font = wxFont(m_mathFontSize, wxFONTFAMILY_DEFAULT,
                   wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL,
                   false, m_mathFontName);
-  
+
   if(!font.IsOk())
     font = *wxNORMAL_FONT;
 
@@ -1365,8 +1337,12 @@ void ConfigDialogue::OnChangeFontFamily(wxCommandEvent &event)
 #endif
     font = *wxNORMAL_FONT;
 
-  if (tmp == &m_styleText || tmp == &m_styleTitle || tmp == &m_styleSubsubsection || tmp == &m_styleSubsection ||
-      tmp == &m_styleSection)
+  if (
+    (tmp == &m_styleText)          ||
+    (tmp == &m_styleTitle)         ||
+    (tmp == &m_styleSubsubsection) ||
+    (tmp == &m_styleSubsection)    ||
+    (tmp == &m_styleSection))
   {
     if (tmp->fontSize != 0)
       fontsize = tmp->fontSize;
@@ -1472,9 +1448,8 @@ void ConfigDialogue::ReadStyles(wxString file)
                    &tmp))
     m_styleOutdated.color.Set(tmp);
 
-  wxSystemSettings settings;
-  m_styleSelection.color = settings.GetColour(wxSYS_COLOUR_HIGHLIGHT);
-  
+  m_styleSelection.color = wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT);
+
   if (config->Read(wxT("Style/Selection/color"),
                    &tmp))
     m_styleSelection.color.Set(tmp);
@@ -1841,7 +1816,7 @@ void ConfigDialogue::OnChangeColor()
   style *tmp = GetStylePointer();
 //  if(tmp->color.alpha == 0)
 //    if(tmp->color.alpha = 10);
-    
+
   wxColour col = wxGetColourFromUser(this, tmp->color);
   if (col.IsOk())
   {
@@ -1860,14 +1835,17 @@ void ConfigDialogue::OnChangeStyle(wxCommandEvent&  WXUNUSED(event))
 
   // MAGIC NUMBERS:
   // the positions of TEXT and TITLE style in the list.
-  if (st >= 15 && st <= 18)
+  if (st >= 14 && st <= 18)
     m_getStyleFont->Enable(true);
   else
     m_getStyleFont->Enable(false);
 
   // Background color only
-  if (st >= TS_SECTION)
+  if (st > TS_TITLE)
   {
+    m_boldCB->SetValue(false);
+    m_italicCB->SetValue(false);
+    m_underlinedCB->SetValue(false);
     m_boldCB->Enable(false);
     m_italicCB->Enable(false);
     m_underlinedCB->Enable(false);
@@ -2026,7 +2004,7 @@ void ConfigDialogue::UpdateExample()
   style *tmp = GetStylePointer();
   if(tmp == NULL)
     return;
-  
+
   wxString example = _("Example text");
   wxColour color(tmp->color);
   wxString font(m_styleDefault.font);
@@ -2135,7 +2113,7 @@ wxPanel(parent, id,
   m_color = wxColour(0, 0, 0);
   m_configDialogue = conf;
   SetBackgroundColour(m_color);
-};
+}
 
 void ConfigDialogue::ColorPanel::OnClick(wxMouseEvent& WXUNUSED(event))
 {
@@ -2165,7 +2143,7 @@ void ConfigDialogue::ColorPanel::OnPaint(wxPaintEvent &WXUNUSED(event))
       if(((x+y)&1) == 1)
         dc.DrawRectangle(x*12,y*12,12,12);
     }
-        
+
   wxColor foregroundColor
     (
       m_color.Red()   * m_color.Alpha() / wxALPHA_OPAQUE + (wxALPHA_OPAQUE - m_color.Alpha()),
@@ -2207,7 +2185,7 @@ void ConfigDialogue::ExamplePanel::OnPaint(wxPaintEvent& WXUNUSED(event))
   wxFont font = wxFont(m_size, wxFONTFAMILY_MODERN, italic, bold, underlined, m_font);
   if(!font.IsOk())
     font = *wxNORMAL_FONT;
-  
+
   font.SetPointSize(m_size);
   if(font.IsOk())
     dc.SetFont(font);
