@@ -18,6 +18,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*! \file
   This file defines the class Configuration which serves as a fast configuration storage.
@@ -34,7 +35,7 @@ Configuration::Configuration(wxDC &dc) : m_dc(&dc)
   m_TOCshowsSectionNumbers = false;
   m_antialiassingDC = NULL;
   m_parenthesisDrawMode = unknown;
-  m_mathJaxURL = wxT("https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.2/MathJax.js?config=TeX-AMS_HTML");
+  m_mathJaxURL = wxT("https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.5/MathJax.js?config=TeX-AMS_HTML");
   m_zoomFactor = 1.0; // affects returned fontsizes
   m_top = -1;
   m_bottom = -1;
@@ -53,11 +54,15 @@ Configuration::Configuration(wxDC &dc) : m_dc(&dc)
   m_hideBrackets = true;
   m_lineWidth_em = 88;
   m_showLabelChoice = 1;
+  m_abortOnError = true;
+  m_autoSaveInterval = 3 * 60 * 1000;
   m_clientWidth = 1024;
   m_clientHeight = 768;
   Dirstructure dirstruct;
+  m_indentMaths=true;
   m_maximaLocation = dirstruct.MaximaDefaultLocation();
   m_indent = -1;
+  m_autoSubscript = 1;
   m_antiAliasLines = true;
   ReadConfig();
   m_showCodeCells = true;
@@ -69,8 +74,25 @@ Configuration::Configuration(wxDC &dc) : m_dc(&dc)
   m_copyMathMLHTML = false;
   m_copyRTF = true;
   m_copySVG = true;
+  m_copyEMF = false;
   m_showLength = 2;
   m_useUnicodeMaths = true;
+  m_defaultBackgroundColor = *wxWHITE;
+}
+
+wxString Configuration::GetAutosubscript_string(){
+  switch (m_autoSubscript)
+  {
+  case 0:
+    return "nil";
+    break;
+  case 1:
+    return "t";
+    break;
+  default:
+    return "'all";
+    break;
+  }
 }
 
 void Configuration::ShowCodeCells(bool show)
@@ -90,6 +112,9 @@ bool Configuration::MaximaFound(wxString location)
   if (wxFileExists(location + wxT("/Contents/Resources/maxima.sh")))
     maximaFound = true;
 
+  // Don't complain if PATH doesn't yield a result.
+  wxLogNull logNull;
+  
   wxPathList pathlist;
   pathlist.AddEnvList(wxT("PATH"));
   wxString path = pathlist.FindAbsoluteValidPath(location);
@@ -104,13 +129,21 @@ void Configuration::ReadConfig()
   wxConfig *config = (wxConfig *) wxConfig::Get();
   m_autoWrap = 3;
 
+  wxString bgColStr = wxT("white");
+  config->Read(wxT("Style/Background/color"), &bgColStr);
+  m_defaultBackgroundColor = wxColour(bgColStr);
+    
+  if(config->Read(wxT("autoSaveInterval"),&m_autoSaveInterval))
+    m_autoSaveInterval *= 1000 * 60;
   config->Read(wxT("TOCshowsSectionNumbers"), &m_TOCshowsSectionNumbers);
   config->Read(wxT("autoWrapMode"), &m_autoWrap);
   config->Read(wxT("useUnicodeMaths"), &m_useUnicodeMaths);
   config->Read(wxT("mathJaxURL"), &m_mathJaxURL);
+  config->Read(wxT("autosubscript"), &m_autoSubscript);
+  config->Read(wxT("antiAliasLines"), &m_antiAliasLines);
+  config->Read(wxT("indentMaths"), &m_indentMaths);
+  config->Read(wxT("abortOnError"),&m_abortOnError);
 
-  config->Read(wxT("antiAliasLines"), & m_antiAliasLines);
-  
   config->Read(wxT("fixReorderedIndices"), &m_fixReorderedIndices);
 
   config->Read(wxT("showLength"), &m_showLength);
@@ -121,6 +154,7 @@ void Configuration::ReadConfig()
   config->Read(wxT("copyMathMLHTML"), &m_copyMathMLHTML);
   config->Read(wxT("copyRTF"), &m_copyRTF);
   config->Read(wxT("copySVG"), &m_copySVG );
+  config->Read(wxT("copyEMF"), &m_copyEMF );
 
   config->Read(wxT("maxima"), &m_maximaLocation);
   // Fix wrong" maxima=1" paraneter in ~/.wxMaxima if upgrading from 0.7.0a
@@ -382,12 +416,16 @@ bool Configuration::CharsExistInFont(wxFont font, wxString char1,wxString char2,
 
 wxString Configuration::GetFontName(int type)
 {
+  wxString retval = m_fontName;
   if (type == TS_TITLE || type == TS_SUBSECTION || type == TS_SUBSUBSECTION || type == TS_SECTION || type == TS_TEXT)
-    return m_styles[type].font;
-  else if (type == TS_NUMBER || type == TS_VARIABLE || type == TS_FUNCTION ||
-           type == TS_SPECIAL_CONSTANT || type == TS_STRING)
-    return m_mathFontName;
-  return m_fontName;
+    retval = m_styles[type].font;
+  if(retval == wxEmptyString)
+    retval = m_fontName;
+  
+  if (type == TS_NUMBER || type == TS_VARIABLE || type == TS_FUNCTION ||
+      type == TS_SPECIAL_CONSTANT || type == TS_STRING)
+    retval = m_mathFontName;
+  return retval;
 }
 
 void Configuration::ReadStyle()
@@ -696,8 +734,7 @@ void Configuration::ReadStyle()
                    &tmp))
     m_styles[TS_CURSOR].color.Set(tmp);
 
-  wxSystemSettings settings;
-  m_styles[TS_SELECTION].color = settings.GetColour(wxSYS_COLOUR_HIGHLIGHT);
+   m_styles[TS_SELECTION].color = wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT);
 
   if (config->Read(wxT("Style/Selection/color"),
                    &tmp))
@@ -758,17 +795,3 @@ int Configuration::Scale_Px(double px)
     retval = 1;
   return retval;
 }
-
-/*
-wxFontEncoding Configuration::GetGreekFontEncoding()
-{
-#if wxUSE_UNICODE || defined (__WXGTK20__) || defined (__WXMAC__)
-  return wxFONTENCODING_DEFAULT;
-#elif defined __WXMSW__
-  return wxFONTENCODING_CP1253;
-#else
-  return wxFONTENCODING_ISO8859_7;
-#endif
-}
-*/
-
