@@ -17,6 +17,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 #ifndef CONFIGURATION_H
 #define CONFIGURATION_H
@@ -27,12 +28,10 @@
 
 #include "TextStyle.h"
 #include "Dirstructure.h"
-#include "Setup.h"
 
 
-#define MC_CELL_SKIP 0
-#define MC_LINE_SKIP 2
-#define MC_TEXT_PADDING 1
+#define MC_LINE_SKIP Scale_Px(2)
+#define MC_TEXT_PADDING Scale_Px(1)
 
 #define PAREN_OPEN_TOP_UNICODE     "\x239b"
 #define PAREN_OPEN_EXTEND_UNICODE  "\x239c"
@@ -87,6 +86,7 @@
 class Configuration
 {
 public:
+  Dirstructure m_dirStructure;
   enum drawMode
   {
     ascii,              //!< Use ascii characters only
@@ -247,7 +247,7 @@ public:
   }
 
   int GetLabelWidth()
-  { return m_labelWidth; }
+  { return m_labelWidth * 14; }
 
   //! Get the indentation of GroupCells.
   int GetIndent()
@@ -330,7 +330,10 @@ public:
   // But text blocks that are 1 meter wide and 2 cm high feel - weird.
   int GetLineWidth()
   {
-    if (m_clientWidth <= m_zoomFactor * double(m_defaultFontSize) * LineWidth_em() * m_zoomFactor)
+    if (
+      (m_clientWidth <= m_zoomFactor * double(m_defaultFontSize) * LineWidth_em() * m_zoomFactor) ||
+      (m_printer)
+      )
       return m_clientWidth;
     else
       return (const int) (double(m_defaultFontSize) * LineWidth_em() * m_zoomFactor);
@@ -370,6 +373,9 @@ public:
     wxConfig::Get()->Write(wxT("autoIndent"), m_autoIndent = autoIndent);
   }
 
+  //! Do we want to indent all maths?
+  bool IndentMaths(){return m_indentMaths;}
+  void IndentMaths(bool indent){wxConfig::Get()->Write(wxT("indentMaths"), m_indentMaths=indent);}
   int GetFontSize(int st)
   {
     if (st == TS_TEXT || st == TS_SUBSUBSECTION || st == TS_SUBSECTION || st == TS_SECTION || st == TS_TITLE)
@@ -577,6 +583,11 @@ public:
     {
       wxConfig::Get()->Write(wxT("copySVG"), m_copySVG = copySVG );
     }
+  bool CopyEMF(){return m_copyEMF;}
+  void CopyEMF(bool copyEMF)
+    {
+      wxConfig::Get()->Write(wxT("copyEMF"), m_copyEMF = copyEMF );
+    }
   void ShowLength(int length)
     {
       wxConfig::Get()->Write(wxT("showLength"), m_showLength = length );
@@ -616,7 +627,34 @@ public:
   wxWindow *GetWorkSheet(){return m_workSheet;}
   //! Set the worksheet this configuration storage is valid for
   void SetWorkSheet(wxWindow *workSheet){m_workSheet = workSheet;}
+
+  //! Get the autosave interval [in milliseconds]; 0 = no autosave
+  int AutoSaveInterval(){if (m_autoSaveInterval > 1) return m_autoSaveInterval; else return 0;}
+  //! Set the autosave interval [in milliseconds]; 0 = noautosave
+  void AutoSaveInterval(int miliseconds){wxConfig::Get()->Write(wxT("autoSaveInterval"),
+                                                  (m_autoSaveInterval = miliseconds / 1000 / 60)
+      );
+  }
+  
+  bool GetAbortOnError(){return m_abortOnError;}
+  void SetAbortOnError(bool abortOnError)
+    {wxConfig::Get()->Write("abortOnError",m_abortOnError = abortOnError);}
+  
+  //! Get the worksheet this configuration storage is valid for
+  int GetAutosubscript_Num(){return m_autoSubscript;}
+  void SetAutosubscript_Num(int autosubscriptnum)
+    {wxConfig::Get()->Write("autosubscript",m_autoSubscript = autosubscriptnum);}
+  wxString GetAutosubscript_string();
+  //! Determine the default background color of the worksheet
+  wxColor DefaultBackgroundColor(){return m_defaultBackgroundColor;}
 private:
+  /*! The interval between auto-saves (in milliseconds). 
+
+    Values <10000 mean: Auto-save is off.
+  */
+  int m_autoSaveInterval;
+  //! Which objects do we want to convert into subscripts if they occur after an underscore?
+  int m_autoSubscript;
   //! The worksheet this configuration storage is valid for
   wxWindow *m_workSheet;
   //! A replacement for the non-existing "==" operator for wxBitmaps.
@@ -702,8 +740,12 @@ private:
   int m_showLength;
   bool m_copyRTF;
   bool m_copySVG;
+  bool m_copyEMF;
   bool m_TOCshowsSectionNumbers;
   bool m_useUnicodeMaths;
+  bool m_indentMaths;
+  bool m_abortOnError;
+  wxColour m_defaultBackgroundColor;
 };
 
 #endif // CONFIGURATION_H
