@@ -19,6 +19,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 #ifndef EDITORCELL_H
 #define EDITORCELL_H
@@ -72,13 +73,11 @@ private:
   //! The font size the linewrap was done for.
   int m_oldDefaultFontSize;
 
-  int ChangeNumpadToChar(int c);
-
   //! A list of all potential autoComplete targets within this cell
   wxArrayString m_wordList;
 
   //! Draw a box that marks the current selection
-  void MarkSelection(long start, long end, wxDC *dc, TextStyle style, int fontsize);
+  void MarkSelection(long start, long end, TextStyle style, int fontsize);
 
   /*! The start of the current selection.
 
@@ -105,6 +104,12 @@ public:
 
   ~EditorCell();
 
+  //! Get the whole maxima command that is currently under the cursor (including all arguments)
+  wxString GetFullCommandUnderCursor();
+
+  //! Add a new parameter to a draw- or similar command including the comma, if needed.
+  void AddDrawParameter(wxString param);
+
   //! May this Editor Cell contain the answer to a question?
   void AutoAnswer(bool autoAnswer){m_autoAnswer = autoAnswer;}
 
@@ -311,17 +316,17 @@ public:
   wxPoint PositionToPoint(int fontsize, int pos = -1);
 
   //! Sets the cursor to the screen coordinate point
-  void SelectPointText(wxDC *dc, wxPoint &point);
+  void SelectPointText(wxPoint &point);
 
   //! Selects the text between the screen coordinates one and two
-  void SelectRectText(wxDC *dc, wxPoint &one, wxPoint &two);
+  void SelectRectText(wxPoint &one, wxPoint &two);
 
   //! Selects the word the cursor is currently at.
   wxString SelectWordUnderCaret(bool selectParens = true, bool toRight = true,
                                 bool includeDoubleQuotes = false);
 
   //! Is the point point inside the currently selected text?
-  bool IsPointInSelection(wxDC *dc, wxPoint point);
+  bool IsPointInSelection(wxPoint point);
 
   bool CopyToClipboard();
 
@@ -371,7 +376,7 @@ public:
 
   void FindMatchingParens();
 
-  int GetLineWidth(wxDC *dc, unsigned int line, int end);
+  int GetLineWidth(unsigned int line, int end);
 
   //! true, if this cell's width has to be recalculated.
   bool IsDirty()
@@ -491,9 +496,10 @@ public:
       - true = we want the new string to be selected afterwards
       - false = the selection is cleared after replacing the string
         and moving the cursor to its end.
-    \param ignoreCase
+    \param ignoreCase true = ignore the case of the string to replace
+    \param replaceMaximaString true = replace strings including the double quotes.
    */
-  bool ReplaceSelection(wxString oldStr, wxString newString, bool keepSelected = false, bool ignoreCase = false);
+  bool ReplaceSelection(wxString oldStr, wxString newString, bool keepSelected = false, bool ignoreCase = false, bool replaceMaximaString = false);
 
   //! Convert the current selection to a string
   wxString GetSelectionString();
@@ -511,6 +517,9 @@ public:
 
   void GotoError(){SetCaretPosition(m_errorIndex);ActivateCursor();}
 
+  //! Start a new line and optionally auto-indent it.
+  void ProcessNewline(bool keepCursorAtStartOfLine = true);
+
   //! Get the cursor's current position inside the cell.
   int GetCaretPosition()
   { return m_positionOfCaret; }
@@ -569,7 +578,7 @@ private:
   bool IsNum(wxChar c);
 
   bool IsAlphaNum(wxChar c);
-  
+
   /*! A piece of styled text for syntax highlighting
 
     A piece of styled text may be
@@ -676,8 +685,6 @@ private:
    */
   int GetIndentDepth(wxString text, int positionOfCaret);
 
-#if wxUSE_UNICODE
-
   /*! Handle ESC shortcuts for special characters
 
     These characters can be tought to LaTeX and the html browser if necessary in
@@ -686,7 +693,6 @@ private:
    */
   wxString InterpretEscapeString(wxString txt);
 
-#endif
   wxString m_text;
   wxArrayString m_textHistory;
   std::vector<int> m_positionHistory;
