@@ -20,6 +20,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*! \file
   This file defines the class GroupCell
@@ -72,7 +73,7 @@ GroupCell::GroupCell(Configuration **config, int groupType, CellPointers *cellPo
     if (groupType == GC_TYPE_CODE)
       m_inputLabel = new TextCell(this, m_configuration, m_cellPointers, EMPTY_INPUT_LABEL);
     else
-      m_inputLabel = new TextCell(this, m_configuration, m_cellPointers, wxT(" "));
+      m_inputLabel = new TextCell(this, m_configuration, m_cellPointers, wxT(""));
 
     m_inputLabel->SetType(MC_TYPE_MAIN_PROMPT);
   }
@@ -574,7 +575,7 @@ void GroupCell::RecalculateWidths(int fontsize)
       // recalculate the position of input in ReEvaluateSelection!
       if (m_inputLabel->m_next != NULL)
       {
-        m_inputLabel->m_next->m_currentPoint.x = m_currentPoint.x + m_inputLabel->GetWidth() + MC_CELL_SKIP;
+        m_inputLabel->m_next->m_currentPoint.x = m_currentPoint.x + m_inputLabel->GetWidth();
       }
     }
     
@@ -588,8 +589,7 @@ void GroupCell::RecalculateWidths(int fontsize)
       else
         m_width = 0;
     }
-    else
-    {
+    else    {
       MathCell *tmp = m_output;
       while (tmp != NULL)
       {
@@ -625,7 +625,9 @@ void GroupCell::InputHeightChanged()
   if (m_inputLabel != NULL) {
     m_inputLabel->ResetData();
   }
-  RecalculateHeight(m_fontSize);
+  RecalculateHeightInput(m_fontSize);
+  if(m_output)
+    m_height += m_output->GetMaxHeight();
 }
 
 // Called on resize events
@@ -688,15 +690,15 @@ void GroupCell::RecalculateHeightInput(int fontsize)
     }
   }
   
+  m_currentPoint.x = configuration->GetIndent();
   if (m_previous == NULL)
   {
-    m_currentPoint.x = configuration->GetIndent();
     m_currentPoint.y = (*m_configuration)->GetBaseIndent() + GetMaxCenter();
   }
   else
     m_currentPoint.y = dynamic_cast<GroupCell *>(m_previous)->m_currentPoint.y +
-    dynamic_cast<GroupCell *>(m_previous)->GetMaxDrop() + GetMaxCenter() +
-    (*m_configuration)->GetGroupSkip();
+      dynamic_cast<GroupCell *>(m_previous)->GetMaxDrop() + GetMaxCenter() +
+      (*m_configuration)->GetGroupSkip();
   
   m_outputRect.x = m_currentPoint.x;
   m_outputRect.y = m_currentPoint.y;
@@ -770,7 +772,11 @@ void GroupCell::RecalculateHeight(int fontsize)
   if (m_inputLabel)
     m_inputLabel->m_currentPoint = m_currentPoint;
   if (GetEditable())
+  {
     GetEditable()->m_currentPoint = m_currentPoint;
+    if(m_inputLabel != NULL)
+      GetEditable()->m_currentPoint.x += m_inputLabel->GetWidth();
+  }
 }
 
 // We assume that appended cells will be in a new line!
@@ -809,7 +815,9 @@ void GroupCell::RecalculateAppended()
   tmp = m_appendedCells;
   while (tmp != NULL)
   {
-    if (tmp->BreakLineHere() || tmp == m_appendedCells || tmp->GetStyle() == TS_LABEL || tmp->GetStyle() == TS_USERLABEL)
+    if ((tmp->BreakLineHere()) ||
+        ((tmp->m_previousToDraw == NULL) && (tmp->m_nextToDraw == NULL)) ||
+        (tmp->GetStyle() == TS_LABEL) || (tmp->GetStyle() == TS_USERLABEL))
     {
       m_width = MAX(m_width, tmp->GetLineWidth());
       m_outputRect.width = MAX(m_outputRect.width, tmp->GetLineWidth());
@@ -876,7 +884,7 @@ void GroupCell::Draw(wxPoint point, int fontsize)
       //
       // Paint background if we have a text cell
       //
-      if (m_groupType == GC_TYPE_TEXT && !configuration->GetPrinter())
+      if (m_groupType == GC_TYPE_TEXT )
       {
         int y = rect.GetY();
 
@@ -921,8 +929,6 @@ void GroupCell::Draw(wxPoint point, int fontsize)
 
         while (tmp != NULL)
         {
-
-
           if (tmp->BreakLineHere())
           {
             if (tmp->m_bigSkip)
@@ -934,7 +940,17 @@ void GroupCell::Draw(wxPoint point, int fontsize)
           }
         
           tmp->m_currentPoint = in;
-        
+
+          if(
+              (tmp->GetStyle() != TS_LABEL) &&
+              (tmp->GetStyle() != TS_USERLABEL) &&
+              (tmp->GetStyle() != TS_MAIN_PROMPT) &&
+              (tmp->GetStyle() != TS_OTHER_PROMPT) &&
+              (tmp->m_previousToDraw == NULL) &&
+              configuration->IndentMaths()
+            )
+            in.x += Scale_Px(configuration->GetLabelWidth()) + MC_TEXT_PADDING;
+          
           if (!tmp->m_isBroken)
           {
             if (tmp->DrawThisCell(in))
@@ -944,11 +960,19 @@ void GroupCell::Draw(wxPoint point, int fontsize)
               if (tmp->m_nextToDraw->BreakLineHere())
               {
                 in.x = point.x;
+                if(
+                  (tmp->m_nextToDraw->GetStyle() != TS_LABEL) &&
+                  (tmp->m_nextToDraw->GetStyle() != TS_USERLABEL) &&
+                  (tmp->m_nextToDraw->GetStyle() != TS_MAIN_PROMPT) &&
+                  (tmp->m_nextToDraw->GetStyle() != TS_OTHER_PROMPT) &&
+                  configuration->IndentMaths()
+                  )
+                  in.x += Scale_Px(configuration->GetLabelWidth()) + MC_TEXT_PADDING;
                 in.y += drop + tmp->m_nextToDraw->GetMaxCenter();
                 drop = tmp->m_nextToDraw->GetMaxDrop();
               }
               else
-                in.x += (tmp->GetWidth() + MC_CELL_SKIP);
+                in.x += (tmp->GetWidth());
             }
 
           }
@@ -957,6 +981,14 @@ void GroupCell::Draw(wxPoint point, int fontsize)
             if (tmp->m_nextToDraw != NULL && tmp->m_nextToDraw->BreakLineHere())
             {
               in.x = point.x;
+              if(
+                (tmp->m_nextToDraw->GetStyle() != TS_LABEL) &&
+                (tmp->m_nextToDraw->GetStyle() != TS_USERLABEL) &&
+                (tmp->m_nextToDraw->GetStyle() != TS_MAIN_PROMPT) &&
+                (tmp->m_nextToDraw->GetStyle() != TS_OTHER_PROMPT) &&
+                configuration->IndentMaths()
+                )
+                in.x += Scale_Px(configuration->GetLabelWidth()) + MC_TEXT_PADDING;
               in.y += drop + tmp->m_nextToDraw->GetMaxCenter();
               drop = tmp->m_nextToDraw->GetMaxDrop();
             }
@@ -974,12 +1006,7 @@ void GroupCell::CellUnderPointer(GroupCell *cell)
 {
   if (m_cellPointers->m_groupCellUnderPointer != cell)
   {
-    GroupCell *tmp = dynamic_cast<GroupCell *>(m_cellPointers->m_groupCellUnderPointer);
     m_cellPointers->m_groupCellUnderPointer = cell;
-    if (tmp != NULL)
-      tmp->DrawBracket();
-    if (m_cellPointers->m_groupCellUnderPointer != NULL)
-      dynamic_cast<GroupCell *>(m_cellPointers->m_groupCellUnderPointer)->DrawBracket();
   }
 }
 
@@ -1278,9 +1305,6 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
   wxASSERT_MSG((imgCounter != NULL), _("Bug: No image counter to write to!"));
   if (imgCounter == NULL) return wxEmptyString;
   wxString str;
-  // Now we might want to introduce some markdown:
-  MarkDownTeX MarkDown(*m_configuration);
-
   switch (m_groupType)
   {
     case GC_TYPE_PAGEBREAK:
@@ -1315,6 +1339,7 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
 
     case GC_TYPE_CODE:
       str = ToTeXCodeCell(imgDir, filename, imgCounter);
+      str.Replace(wxT("\\[\\displaystyle \\]"),wxT(""));
       break;
 
     default:
@@ -1343,10 +1368,6 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
               str = GetEditable()->ToString();
               str = str.Mid(5, str.Length());
             }
-            else
-            {
-              str = MarkDown.MarkDown(str);
-            }
             break;
         }
       }
@@ -1359,15 +1380,19 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
 wxString GroupCell::ToTeXCodeCell(wxString imgDir, wxString filename, int *imgCounter)
 {
   wxString str;
+  Configuration *configuration = (*m_configuration);
 
   // Input cells
-  if ((*m_configuration)->ShowCodeCells())
+  if (configuration->ShowCodeCells())
   {
     str = wxT("\n\n\\noindent\n%%%%%%%%%%%%%%%\n")
-                  wxT("%%% INPUT:\n")
-                  wxT("\\begin{minipage}[t]{8ex}\\color{red}\\bf\n") +
-          m_inputLabel->ToTeX() +
-          wxT("\n\\end{minipage}");
+      wxT("%%% INPUT:\n") +
+      wxString::Format(
+        wxT("\\begin{minipage}[t]{%iem}\\color{red}\\bf\n"),
+        configuration->GetLabelWidth()/14
+        ) +
+      m_inputLabel->ToTeX() +
+      wxT("\n\\end{minipage}");
     if (m_inputLabel->m_next != NULL)
     {
 
@@ -1405,10 +1430,10 @@ wxString GroupCell::ToTeXCodeCell(wxString imgDir, wxString filename, int *imgCo
           case TS_LABEL:
           case TS_USERLABEL:
             if (mathMode)
-              str += wxT("\\mbox{}\\]\n\\[\\displaystyle\n");
+              str += wxT("\\mbox{}\\]\n\\[\\displaystyle ");
             else
             {
-              str += wxT("\\[\\displaystyle\n");
+              str += wxT("\\[\\displaystyle ");
               mathMode = true;
             }
             str += tmp->ToTeX() + wxT("\n");
@@ -1426,7 +1451,7 @@ wxString GroupCell::ToTeXCodeCell(wxString imgDir, wxString filename, int *imgCo
           default:
             if (!mathMode)
             {
-              str += wxT("\\[\\displaystyle\n");
+              str += wxT("\\[\\displaystyle ");
               mathMode = true;
             }
             str += tmp->ToTeX();
@@ -1444,6 +1469,8 @@ wxString GroupCell::ToTeXCodeCell(wxString imgDir, wxString filename, int *imgCo
       str += wxT("\\mbox{}\n\\]\n%%%%%%%%%%%%%%%");
     }
   }
+  else
+    str+=wxT("\n\n\\noindent%\n");
 
   return str;
 }
@@ -1863,20 +1890,36 @@ void GroupCell::BreakUpCells(int fontsize, int clientWidth)
 
 void GroupCell::BreakUpCells(MathCell *cell, int WXUNUSED(fontsize), int clientWidth)
 {
+  if(cell == NULL)
+    return;
+
   MathCell *tmp = cell;
 
+  bool lineHeightsChanged = false;
+    
   while (tmp != NULL && !m_hide)
   {
     if (tmp->GetWidth() > clientWidth)
     {
       if (tmp->BreakUp())
-      {
-        tmp->RecalculateWidths(tmp->IsMath() ? m_mathFontSize : m_fontSize);
-        tmp->RecalculateHeight(tmp->IsMath() ? m_mathFontSize : m_fontSize);
-      }
+        lineHeightsChanged = true;
     }
     tmp = tmp->m_nextToDraw;
   }
+
+  if(lineHeightsChanged)
+  {
+    if(m_inputLabel != NULL)
+    {
+      m_inputLabel->RecalculateList((*m_configuration)->GetDefaultFontSize());
+    }
+    if(m_output != NULL)
+    {
+      m_output->ResetSizeList();
+      m_output->RecalculateList(m_output->IsMath() ? m_mathFontSize : m_fontSize);
+    }
+    ResetData();
+  }
 }
 
 void GroupCell::UnBreakUpCells()
@@ -2007,16 +2050,19 @@ GroupCell *GroupCell::Fold()
 
   // cell(s) to fold are between start and end (including these two)
 
-  if(end->m_next != NULL)
+  if(end != NULL)
   {
-    m_next = m_nextToDraw = end->m_next;
-    end->m_next->m_previous = end->m_next->m_previousToDraw = this;
+    if(end->m_next != NULL)
+    {
+      m_next = m_nextToDraw = end->m_next;
+      end->m_next->m_previous = end->m_next->m_previousToDraw = this;
+    }
+    else
+      m_next = m_nextToDraw = NULL;
+    end->m_next = end->m_nextToDraw = NULL;
   }
-  else
-    m_next = m_nextToDraw = NULL;
   
   start->m_previous = start->m_previousToDraw = NULL;
-  end->m_next = end->m_nextToDraw = NULL;
   m_hiddenTree = start; // save the torn out tree into m_hiddenTree
   m_hiddenTree->SetHiddenTreeParent(this);
   return this;
@@ -2205,33 +2251,33 @@ bool GroupCell::Contains(GroupCell *cell)
 #if wxUSE_ACCESSIBILITY
 wxAccStatus GroupCell::GetDescription(int childId, wxString *description)
 {
-	if (description == NULL)
-		return wxACC_FAIL;
-
-	if (childId == 0)
-	{
-		if (m_groupType == GC_TYPE_PAGEBREAK)
-		{
-			*description = _("A page break");
-			return wxACC_OK;
-		}
-		else
-		{
-			*description = _("A GroupCell that bundles input with its output");
-			return wxACC_OK;
-		}
-	}
-	else
-	{
-		MathCell *cell = NULL;
-		if (GetChild(childId, &cell) == wxACC_OK)
-		{
-			return cell->GetDescription(0, description);
-		}
-	}
-
-	*description = wxEmptyString;
-	return wxACC_FAIL;
+  if (description == NULL)
+    return wxACC_FAIL;
+  
+  if (childId == 0)
+  {
+    if (m_groupType == GC_TYPE_PAGEBREAK)
+    {
+      *description = _("A page break");
+      return wxACC_OK;
+    }
+    else
+    {
+      *description = _("A GroupCell that bundles input with its output");
+      return wxACC_OK;
+    }
+  }
+  else
+  {
+    MathCell *cell = NULL;
+    if (GetChild(childId, &cell) == wxACC_OK)
+    {
+      return cell->GetDescription(0, description);
+    }
+  }
+  
+  *description = wxEmptyString;
+  return wxACC_FAIL;
 }
 
 
@@ -2241,10 +2287,10 @@ wxAccStatus GroupCell::GetLocation(wxRect &rect, int elementId)
   {
     rect = wxRect(GetRect().GetTopLeft()     + m_visibleRegion.GetTopLeft(),
                   GetRect().GetBottomRight() + m_visibleRegion.GetTopLeft());
-
+    
     // Our GroupCell handles the hcaret below the cell, as well as its contents
     rect.SetBottom(rect.GetBottom()+(*m_configuration)->GetGroupSkip());
-
+    
     // If we are the 1st groupcell of the worksheet we handle the hcaret above this
     // cell, too.
     if(m_previous == NULL)
