@@ -18,6 +18,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*! \file
   This file defines the class MathCell
@@ -34,7 +35,7 @@ wxString MathCell::GetToolTip(const wxPoint &point)
   if(!ContainsPoint(point))
     return wxEmptyString;
 
-  wxString toolTip;  
+  wxString toolTip;
   std::list<MathCell *> innerCells = GetInnerCells();
   for(std::list<MathCell *>::iterator it = innerCells.begin(); it != innerCells.end(); ++it)
   {
@@ -50,6 +51,7 @@ wxString MathCell::GetToolTip(const wxPoint &point)
 MathCell::MathCell(MathCell *group, Configuration **config)
 {
   m_toolTip = wxEmptyString;
+  m_cellPointers = NULL;
   m_group = group;
   m_parent = group;
   m_configuration = config;
@@ -221,7 +223,7 @@ MathCell *MathCell::GetGroup()
  */
 int MathCell::GetMaxCenter()
 {
-  if (m_maxCenter < 0)
+  if ((m_maxCenter < 0) || ((*m_configuration)->ForceUpdate()))
   {
     MathCell *tmp = this;
     while (tmp != NULL)
@@ -242,7 +244,7 @@ int MathCell::GetMaxCenter()
  */
 int MathCell::GetMaxDrop()
 {
-  if (m_maxDrop < 0)
+  if ((m_maxDrop < 0) || ((*m_configuration)->ForceUpdate()))
   {
     MathCell *tmp = this;
     while (tmp != NULL)
@@ -269,17 +271,17 @@ int MathCell::GetMaxHeight()
 int MathCell::GetFullWidth()
 {
   // Recalculate the with of this list of cells only if this has been marked as necessary.
-  if (m_fullWidth < 0)
+  if ((m_fullWidth < 0) || ((*m_configuration)->ForceUpdate()))
   {
     MathCell *tmp = this;
 
     // We begin this calculation with a negative offset since the full width of only a single
     // cell doesn't contain the space that separates two cells - that is automatically added
     // to every cell in the next step.
-    m_fullWidth = -Scale_Px(MC_CELL_SKIP);
+    m_fullWidth = 0;
     while (tmp != NULL)
     {
-      m_fullWidth += tmp->m_width + Scale_Px(MC_CELL_SKIP);
+      m_fullWidth += tmp->m_width;
       tmp = tmp->m_next;
     }
   }
@@ -297,10 +299,9 @@ int MathCell::GetLineWidth()
 
     MathCell *tmp = this;
     while(tmp != NULL)
-    {      
+    {
       width += tmp->m_width;
-      width += Scale_Px(MC_CELL_SKIP);
-      
+
       if (width > m_lineWidth)
         m_lineWidth = width;
 
@@ -338,7 +339,7 @@ void MathCell::DrawList(wxPoint point, int fontsize)
   while (tmp != NULL)
   {
     tmp->Draw(point, fontsize);
-    point.x += tmp->m_width + Scale_Px(MC_CELL_SKIP);
+    point.x += tmp->m_width;
     wxASSERT(tmp != tmp->m_nextToDraw);
     tmp = tmp->m_nextToDraw;
   }
@@ -379,11 +380,11 @@ void MathCell::RecalculateHeightList(int fontsize)
   }
 }
 
-/*! Recalculate widths of cells. 
+/*! Recalculate widths of cells.
 
-  (Used for changing font size since in this case all size information has to be 
+  (Used for changing font size since in this case all size information has to be
   recalculated).
-  
+
   Should set: set m_width.
 */
 void MathCell::RecalculateWidthsList(int fontsize)
@@ -408,7 +409,7 @@ bool MathCell::DrawThisCell(wxPoint point)
 {
   if((point.x < 0) || (point.y < 0))
     return false;
-  
+
   Configuration *configuration = (*m_configuration);
   int top = configuration->GetTop();
   int bottom = configuration->GetBottom();
@@ -423,7 +424,7 @@ bool MathCell::DrawThisCell(wxPoint point)
 
   \param all
    - true  return the rectangle around the whole line.
-   - false return the rectangle around this cell. 
+   - false return the rectangle around this cell.
  */
 wxRect MathCell::GetRect(bool all)
 {
@@ -519,7 +520,7 @@ wxString MathCell::ListToString()
       //    (tmp->GetStyle() != TS_USERLABEL) &&
       //    (tmp->GetStyle() != TS_MAIN_PROMPT) &&
       //    (tmp->GetStyle() != TS_OTHER_PROMPT))
-      //   retval += wxT("\t");        
+      //   retval += wxT("\t");
     }
     // if(firstline)
     // {
@@ -527,10 +528,10 @@ wxString MathCell::ListToString()
     //      (tmp->GetStyle() != TS_USERLABEL) &&
     //      (tmp->GetStyle() != TS_MAIN_PROMPT) &&
     //      (tmp->GetStyle() != TS_OTHER_PROMPT))
-    //     retval += wxT("\t");        
+    //     retval += wxT("\t");
     // }
     retval += tmp->ToString();
-    
+
     firstline = false;
     tmp = tmp->m_nextToDraw;
   }
@@ -950,7 +951,7 @@ void MathCell::SelectInner(wxRect &rect, MathCell **first, MathCell **last)
       if ((*it)->ContainsRect(rect))
         (*it)->SelectRect(rect, first, last);
     }
-  
+
   if (*first == NULL || *last == NULL)
   {
     *first = this;
@@ -979,7 +980,7 @@ bool MathCell::ContainsRect(const wxRect &sm, bool all)
 
  Resets cached data like width and the height of the current cell
  as well as the vertical position of the center. Temporarily unbreaks all
- lines until the widths are recalculated if there aren't any hard line 
+ lines until the widths are recalculated if there aren't any hard line
  breaks.
  */
 void MathCell::ResetData()
@@ -987,7 +988,7 @@ void MathCell::ResetData()
   m_fullWidth = -1;
   m_lineWidth = -1;
   m_maxCenter = -1;
-  m_maxDrop = -1;
+  m_maxDrop   = -1;
   m_breakLine = m_forceBreakLine;
 }
 
@@ -1037,7 +1038,7 @@ void MathCell::SetPen(double lineWidth)
   wxDC *dc = configuration->GetDC();
 
   wxPen pen;
-  
+
   if (m_highlight)
     pen = *(wxThePenList->FindOrCreatePen(configuration->GetColor(TS_HIGHLIGHT),
                                           lineWidth * configuration->GetDefaultLineWidth(), wxPENSTYLE_SOLID));
@@ -1129,7 +1130,7 @@ wxAccStatus MathCell::GetDescription(int childId, wxString *description)
 {
   if(description == NULL)
     return wxACC_FAIL;
-  
+
   if (childId == 0)
   {
 	  *description = _("Math output");
@@ -1153,7 +1154,7 @@ wxAccStatus MathCell::GetParent (wxAccessible **parent)
 {
   if(parent == NULL)
     return wxACC_FAIL;
-  
+
   if(*parent != this)
     *parent = m_parent;
   else
@@ -1168,7 +1169,7 @@ wxAccStatus MathCell::GetValue (int childId, wxString *strValue)
 {
   if(strValue == NULL)
     return wxACC_FAIL;
-  
+
   MathCell *cell;
   if(GetChild(childId,&cell) == wxACC_OK)
   {
@@ -1232,7 +1233,7 @@ wxAccStatus MathCell::GetChild(int childId, MathCell  **child)
 {
   if(child == NULL)
     return wxACC_FAIL;
-  
+
   if (childId == 0)
   {
     *child = this;
@@ -1259,23 +1260,23 @@ wxAccStatus MathCell::GetFocus (int *childId, MathCell  **child)
 {
   int childCount;
   GetChildCount(&childCount);
-  
+
   for(int i = 0; i < childCount;i++)
   {
-    int dummy1; 
+    int dummy1;
     MathCell *cell = NULL;
     GetChild(i + 1, &cell);
     if (cell != NULL)
       if(cell->GetFocus(&dummy1, child) == wxACC_OK)
       {
-        if(childId != NULL)      
+        if(childId != NULL)
           *childId = i+1;
         if(child != NULL)
           *child = cell;
         return wxACC_OK;
       }
   }
-  
+
   if(childId != NULL)
     *childId = 0;
   if(child != NULL)
@@ -1309,19 +1310,21 @@ wxAccStatus MathCell::GetLocation(wxRect &rect, int elementId)
   return wxACC_FAIL;
 }
 
-wxAccStatus MathCell::GetRole (int childId, wxAccRole *role)
+wxAccStatus MathCell::GetRole (int WXUNUSED(childId), wxAccRole *role)
 {
   if(role != NULL)
   {
     *role =   wxROLE_SYSTEM_STATICTEXT;
     return wxACC_OK;
   }
+  else return wxACC_FAIL;
 }
 
 #endif
 
 MathCell::CellPointers::CellPointers(wxScrolledCanvas *mathCtrl)
 {
+  m_wxmxImgCounter = 0;
   m_mathCtrl = mathCtrl;
   m_cellMouseSelectionStartedIn = NULL;
   m_cellKeyboardSelectionStartedIn = NULL;
@@ -1339,6 +1342,13 @@ MathCell::CellPointers::CellPointers(wxScrolledCanvas *mathCtrl)
   m_currentTextCell = NULL;
 }
 
+wxString MathCell::CellPointers::WXMXGetNewFileName()
+{
+  wxString file(wxT("image"));
+  file << (++m_wxmxImgCounter) << wxT(".");
+  return file;
+}
+
 bool MathCell::CellPointers::ErrorList::Contains(MathCell *cell)
 {
   for(std::list<MathCell *>::iterator it = m_errorList.begin(); it != m_errorList.end();++it)
@@ -1360,7 +1370,7 @@ void MathCell::MarkAsDeleted()
     m_cellPointers->m_activeCell = NULL;
   if(this == m_cellPointers->m_currentTextCell)
     m_cellPointers->m_currentTextCell = NULL;
-  
+
   if((this == m_cellPointers->m_selectionStart) || (this == m_cellPointers->m_selectionEnd))
     m_cellPointers->m_selectionStart = m_cellPointers->m_selectionEnd = NULL;
   if(this == m_cellPointers->m_cellUnderPointer)
