@@ -19,6 +19,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*! \file
   This file defines the class MathCtrl
@@ -48,17 +49,17 @@
 
 /*! The canvas that contains the spreadsheet the whole program is about.
 
-This canvas contains all the math-, title-, image- input- ("editor-")- etc.- 
+This canvas contains all the math-, title-, image- input- ("editor-")- etc.-
 cells of the current session.
 
 Most of the logic that handles that this canvas might be larger than the screen
 (and could even be larger than the computer's RAM would it have been handled as
 a big bitmap) is provided by wxWidgets:
- - If part of the screen needs to be drawn it calls the OnPaint() method. 
- - If the canvas is scrolled wxWidgets the parts that are already decides how much 
+ - If part of the screen needs to be drawn it calls the OnPaint() method.
+ - If the canvas is scrolled wxWidgets the parts that are already decides how much
    of the off-screen part of the canvas is cached in a backing store
  - Also if the canvas is scrolled wxWidgets is intelligent enough to only request
-   the parts of the new view that aren't available on the screen or in the backing 
+   the parts of the new view that aren't available on the screen or in the backing
    store
  - If we call the Refresh() method wxWidgets requests us to draw the entire visible
    part of the worksheet anew and invalidates all cached areas it might have.
@@ -71,7 +72,7 @@ wxMaxima has caught up with all the events (which causes wxWidgets to send a Idl
 event to the wxMaxima object) or until we reach a timeout. If the user manages
 to type faster than wxMaxima redraws the screen this allows us to skip
 a few redraws in order to process the keypresses as fast as the user types.
-Also this keeps us responsive even if maxima outputs data faster than 
+Also this keeps us responsive even if maxima outputs data faster than
 wxMaxima can display it.
 */
 class MathCtrl : public wxScrolledCanvas
@@ -82,13 +83,13 @@ private:
   //! The rectangle we need to refresh. -1 as "left" coordinate means: No rectangle
   wxRect m_rectToRefresh;
   /*! The size of a scroll step
-    
-    Defines the size of a 
+
+    Defines the size of a
     scroll step, but besides that also the accuracy wxScrolledCanvas
     calculates some widths in.
   */
   int m_scrollUnit;
-  /*! The drawing contect used for calculating sizes. 
+  /*! The drawing contect used for calculating sizes.
 
     Drawing is done from a wxPaintDC in OnPaint() instead.
   */
@@ -182,7 +183,7 @@ private:
   long m_lastBottom;
   /*! \defgroup UndoBufferFill Undo methods for cell additions/deletions:
 
-    Each EditorCell has its own private undo buffer Additionally wxMaxima 
+    Each EditorCell has its own private undo buffer Additionally wxMaxima
     maintains a undo buffer for worksheet changes. It works the following way:
 
      - Cells normally aren't deleted. They are moved into an undo buffer instead.
@@ -190,7 +191,7 @@ private:
      - If the cursor enters a cell the contents of this cell is saved
      - And if the cell is left and the contents has changed this information
        is kept in the undo buffer, as well.
-     
+
     The add and delete actions offer to choose which undo buffer to use since
     there are two of them:
      - The normal undo buffer and
@@ -198,7 +199,7 @@ private:
        undo command so the undo action can be reverted.\n
        This approach allows the undo functionality to use the regular add and
        delete operations keeping the codebase small.
-    @{ 
+    @{
   */
 
   //! The description of one action for the undo (or redo) command
@@ -266,7 +267,7 @@ private:
 
   //! The text the TreeUndo_ActiveCell contained when we entered it
   wxString m_treeUndo_ActiveCellOldText;
-  
+
   //! Clear the list of actions for which an undo can be undone
   void TreeUndo_ClearRedoActionList();
 
@@ -286,7 +287,7 @@ private:
   //! Add another action to this undo action
   void TreeUndo_AppendAction(){TreeUndo_AppendAction(&treeUndoActions);}
 
-  /*! The last cell we have entered. 
+  /*! The last cell we have entered.
 
     This pointer is needed for keeping track of cell contents changes.
    */
@@ -312,7 +313,7 @@ private:
   bool TreeUndoCellDeletion(std::list<TreeUndoAction *> *actionlist, std::list<TreeUndoAction *> *undoForThisOperation);
   /*! Undo adding cells
 
-    Called from TreeUndo().*/  
+    Called from TreeUndo().*/
   bool TreeUndoCellAddition(std::list<TreeUndoAction *> *actionlist, std::list<TreeUndoAction *> *undoForThisOperation);
 
   //! Undo a tree operation.
@@ -396,25 +397,26 @@ private:
 
     \param start The cell to start copying at
     \param end   The cell the copy has to end with
-    \param asData 
+    \param asData
       - true:  The cells are copied in the order they are stored. m_next and m_previous
                therefore point to the right places. But m_nextToDraw and m_previousToDraw
                will be treated as aliasses of m_next and m_previous.
       - false: If a cell is broken into individual lines m_nextToDraw won't point to the
                next cell that is to be displayed. It will point to the cell containing the
-               function name instead that is followed by the cell containing its contents. 
+               function name instead that is followed by the cell containing its contents.
                This is accurately copied if asdata=false. But m_next and m_previous are
                treated as mere aliasses of m_nextToDraw and m_previousToDraw in this case.
   */
   MathCell *CopySelection(MathCell *start, MathCell *end, bool asData = false);
 
+  //! Get the cordinates of the bottom right point of the worksheet.
   void GetMaxPoint(int *width, int *height);
 
   //! Is executed if a timer associated with MathCtrl has expired.
   void OnTimer(wxTimerEvent &event);
 
   /*! Has the autosave interval expired?
-  
+
     True means: A save will be issued after the user stops typing.
    */
   bool m_autoSaveIntervalExpired;
@@ -428,10 +430,10 @@ private:
 
   void OnMouseEnter(wxMouseEvent &event);
 
-  /*! Is called by wxWidgets when it wants to redraw the console.
+  /*! Is called by wxWidgets when it wants to redraw the worksheet or a part of it.
 
-    The canonical way to trigger this function is calling the Refresh() function
-    of this class.
+    The canonical way to schedule triggering this function is calling the Refresh()
+    function of this class.
    */
   void OnPaint(wxPaintEvent &event);
 
@@ -441,7 +443,7 @@ private:
 
   void OnMouseLeftUp(wxMouseEvent &event);
 
-  //! We lost the mouse connection during drag-and-drop
+  //! Is called if we loose the mouse connection whilst selecting text/cells
   void OnMouseCaptureLost(wxMouseCaptureLostEvent &event);
 
   void OnMouseLeftDown(wxMouseEvent &event);
@@ -456,6 +458,7 @@ private:
 
   void OnMouseWheel(wxMouseEvent &event);
 
+  //! Is called on double click on a cell.
   void OnDoubleClick(wxMouseEvent &event);
 
   //! Key pressed inside a cell
@@ -469,7 +472,7 @@ private:
 
   /*! Handle selecting text using the keyboard
  Is called when the all of the following is true:
-  - We have a wxKeyEvent with no active editor, 
+  - We have a wxKeyEvent with no active editor,
   - shift is down and
   - keycode (ccode) is WXK_UP/WXK_DOWN
  */
@@ -519,10 +522,10 @@ private:
   wxPoint m_down;
   wxPoint m_up;
   wxPoint m_mousePoint;
-  /*! Is the active cursor the one represented by a horizontal line? 
+  /*! Is the active cursor the one represented by a horizontal line?
 
     See m_hCaretPosition and EditorCell::GetActiveCell() for the position of the two
-    types of cursors. 
+    types of cursors.
    */
   bool m_hCaretActive;
   /*! The group above the hcaret, NULL for the top of the document
@@ -532,13 +535,13 @@ private:
   GroupCell *m_hCaretPosition;
   /*! The start for the selection when selecting group with the horizontally drawn cursor
 
-    This cell does define were the selection was actually started and therefore does not need 
+    This cell does define were the selection was actually started and therefore does not need
     to be above m_hCaretPositionEnd in the worksheet. See also m_cellPointers.m_selectionStart.
    */
   GroupCell *m_hCaretPositionStart;
   /*! The end of the selection when selecting group with the horizontally drawn cursor
 
-    This cell does define where the selection was actually ended and therefore does not need 
+    This cell does define where the selection was actually ended and therefore does not need
     to be below m_hCaretPositionEnd in the worksheet. See also m_cellPointers.m_selectionEnd.
    */
   GroupCell *m_hCaretPositionEnd;
@@ -563,7 +566,7 @@ private:
   wxBitmap m_memory;
   //! True if no changes have to be saved.
   bool m_saved;
-  AutoComplete m_autocomplete;
+  AutoComplete *m_autocomplete;
   wxArrayString m_completions;
   bool m_autocompleteTemplates;
   AutocompletePopup *m_autocompletePopup;
@@ -583,7 +586,7 @@ public:
 
   /*! Key for a printable character pressed.
 
-    Can call OnCharInActive or OnCharNoActive, if appropriate. See OnKeyDown for 
+    Can call OnCharInActive or OnCharNoActive, if appropriate. See OnKeyDown for
     non-printable characters like "up" or "right".
    */
   void OnChar(wxKeyEvent &event);
@@ -605,12 +608,12 @@ public:
   void WindowActive(bool active){m_windowActive = active;}
   //! Clears the notification message from SetNotification
   void ClearNotification();
-  /*! Inform the user that something happened in a non-active window 
+  /*! Inform the user that something happened in a non-active window
 
     This command will be ignored if the wxMaxima window is currently active
    */
   void SetNotification(wxString message, int flags = wxICON_INFORMATION);
-  
+
   //! Is called if this element looses or gets the focus
   void OnActivate(wxActivateEvent &event);
   //! The central settings storage
@@ -668,16 +671,16 @@ public:
   }
 
   /*! Handle redrawing the worksheet or of parts of it
-    
+
     This functionality is important for scrolling, if we have changed anything
-    that needs updating or if part of the screen memory containing a window was  
+    that needs updating or if part of the screen memory containing a window was
     overwritten by a window that was in front of it and now needs to be redrawn
     for this reason.
 
     We don't redraw the window immediately if this seems necessary but wait
-    for the idle loop instead - which is called when all selected gui events 
-    have been processed. This allows us to handle events as fast as the user 
-    can type and (if the user types faster than we can display the text) 
+    for the idle loop instead - which is called when all selected gui events
+    have been processed. This allows us to handle events as fast as the user
+    can type and (if the user types faster than we can display the text)
     enables us to update the display as often as the typing speed allows, but
     not more often.
     @{
@@ -697,7 +700,7 @@ public:
 
   /*! Request the worksheet to be redrawn
 
-    \param start Which cell do we need to start the redraw in? Subsequent calls to 
+    \param start Which cell do we need to start the redraw in? Subsequent calls to
     this function with different cells start the redraw at the upmost of the cells
     that were passed to it.
 
@@ -710,7 +713,7 @@ public:
   void RequestRedraw(GroupCell *start = NULL);
   /*! Request a part of the worksheet to be redrawn
 
-    \param rect The rectangle that is to be requested to be redrawn. If this 
+    \param rect The rectangle that is to be requested to be redrawn. If this
     function is called multiple times the rectangles are automatically merged.
 
     The actual redraw is done in the idle loop which means that as many redraw
@@ -741,7 +744,7 @@ public:
   wxString m_currentFile;
 
   /*! Make a few unicode characters interpretable by maxima.
-    
+
     Does convert the not equal sign to a '#' and similar.
 
     \todo We perhaps could think of only doing this outside of strings.
@@ -759,7 +762,7 @@ public:
   FindReplaceDialog *m_findDialog;
 
   /*! True = schedule an update of the table of contents
-    
+
     used by UpdateTableOfContents() and the idle task.
   */
   bool m_scheduleUpdateToc;
@@ -769,8 +772,8 @@ public:
   { return m_hCaretActive; }
 
   /*! Can we merge the selected cells into one?
-    
-    \todo Does it make sense to make to allow the text of sections and image cells 
+
+    \todo Does it make sense to make to allow the text of sections and image cells
     with math cells?
    */
   bool CanMergeSelection();
@@ -809,7 +812,9 @@ public:
     popid_comment_selection,
     popid_divide_cell,
     popid_copy_image,
+    popid_copy_animation,
     popid_copy_svg,
+    popid_copy_emf,
     popid_copy_rtf,
     popid_delete,
     popid_simplify,
@@ -831,6 +836,7 @@ public:
     popid_copy_mathml,
     popid_image,
     popid_svg,
+    popid_emf,
     popid_animation_save,
     popid_animation_start,
     popid_evaluate,
@@ -842,6 +848,7 @@ public:
     popid_insert_subsection,
     popid_insert_subsubsection,
     popid_auto_answer,
+    popid_popup_gnuplot,
     menu_zoom_in,
     menu_zoom_out,
     popid_fold,
@@ -867,9 +874,9 @@ public:
   /*! Insert group cells into the worksheet
 
     \param cells The list of cells that has to be inserted
-    \param where The cell the cells have to be inserted after. NULL means: 
+    \param where The cell the cells have to be inserted after. NULL means:
            Insert the cells at the beginning of the worksheet.
-    \param undoBuffer The buffer the undo information for this action has 
+    \param undoBuffer The buffer the undo information for this action has
            to be kept in. Might be
             - treeUndoActions for normal deletes,
             - treeRedoActions for deletions while executing an undo or
@@ -894,13 +901,16 @@ public:
   */
   void InsertLine(MathCell *newLine, bool forceNewLine = false);
 
-  //! Recalculate the worksheet starting with the cell start.
+  // Actually recalculate the worksheet.
+  bool RecalculateIfNeeded();
+
+  //! Schedule a recalculation of the worksheet starting with the cell start.
   void Recalculate(GroupCell *start, bool force = false);
 
   void Recalculate(bool force = false)
   { Recalculate(m_tree, force); }
 
-  //! Force a full recalculation of the worksheet
+  //! Schedule a full recalculation of the worksheet
   void RecalculateForce()
   {
     Recalculate(true);
@@ -943,7 +953,7 @@ public:
   }
 
   /*! Delete a range of cells
-  
+
     \param start The first cell to delete
     \param end The last cell to delete
     \param undoBuffer The buffer the undo information has to be kept in. Might be
@@ -959,7 +969,7 @@ public:
   );
 
   /*! Move a range of cells from the document to the undo buffer
-  
+
     \param start The first cell to delete
     \param end The last cell to delete
     \addtogroup UndoBufferFill
@@ -970,7 +980,7 @@ public:
   );
 
   /*! Delete the currently selected cells
-    
+
     Actually moves them to the undo buffer so this action can be undone.
     \addtogroup UndoBufferFill
   */
@@ -988,7 +998,7 @@ public:
    */
   void DeleteCurrentCell();
 
-  //! Does it make sense to enable the "Play" button and the slider now? 
+  //! Does it make sense to enable the "Play" button and the slider now?
   bool CanAnimate()
   {
     return m_cellPointers.m_selectionStart != NULL && m_cellPointers.m_selectionStart == m_cellPointers.m_selectionEnd &&
@@ -997,7 +1007,7 @@ public:
 
   /*! Animate the current slide show
 
-    \param run 
+    \param run
       - false: Stop the animation
       - true: Run the animation
   */
@@ -1041,9 +1051,17 @@ public:
   //! Copy a bitmap of the current selection to the clipboard
   bool CopyBitmap();
 
+  //! Copy the current animation to the clipboard
+  bool CopyAnimation();
+
     //! Copy a svg of the current selection to the clipboard
   bool CopySVG();
 
+#if wxUSE_ENH_METAFILE
+  //! Copy a svg of the current selection to the clipboard
+  bool CopyEMF();
+#endif
+
   //! Copy a rtf version of the current selection to the clipboard
   bool CopyRTF();
 
@@ -1068,7 +1086,7 @@ public:
 
   /*! export to xml compatible file
     \param file The file name
-    \param markAsSaved false means that this action doesn't clear the 
+    \param markAsSaved false means that this action doesn't clear the
                              worksheet's "modified" status.
   */
   bool ExportToWXMX(wxString file, bool markAsSaved = true);
@@ -1113,6 +1131,8 @@ public:
   //! Select the cell range start-end
   void SetSelection(MathCell *start, MathCell *end);
 
+  /*! We can edit the input if the we have the whole input in selection!
+   */
   bool CanEdit();
 
   bool ActivatePrevInput();
@@ -1135,11 +1155,11 @@ public:
   GroupCell *FirstVisibleGC();
 
   /*! Scrolls to a point on the worksheet
-  
+
     \todo I have deactivated this assert for the release as it scares the users
     in a case we don't seem to have a problem. But we perhaps should try to find
     out why it is triggered.
-    
+
     Test case:
       - Create a code cell
       - Press the "hide all code cells" button.
@@ -1216,10 +1236,10 @@ public:
   /*! Do we want to follow the evaluation process?
 
     Maxima can automagically scroll to the cell that is currently evaluated.
-    If it does do so can be queried by FollowEvaluation(). Changing the 
+    If it does do so can be queried by FollowEvaluation(). Changing the
     behavior (for example because the user has scrolled away from the
     cell being evaluated and now clearly wants the cursor to stay where
-    it is) can be archieved by FollowEvaluation(true) or 
+    it is) can be archieved by FollowEvaluation(true) or
     FollowEvaluation(false).
    */
   void FollowEvaluation(bool FollowEvaluation);
@@ -1252,7 +1272,7 @@ public:
   // methods related to evaluation queue
 
   /*! Trigger the evaluation of the current cell(s)
-  
+
     Internally this function simulates a click on the "Cell/Evaluate Cell(s)"
     button.
    */
@@ -1271,7 +1291,7 @@ public:
 
   //! Add all cells below the cursor to the evaluation queue.
   void AddRestToEvaluationQueue();
-  
+
   //! Adds a chapter, a section or a subsection to the evaluation queue
   void AddSectionToEvaluationQueue(GroupCell *start);
 
@@ -1341,17 +1361,22 @@ public:
   wxString GetOutputAboveCaret();
 
   bool LoadSymbols()
-  { return m_autocomplete.LoadSymbols(); }
+  { return m_autocomplete->LoadSymbols(); }
 
   bool Autocomplete(AutoComplete::autoCompletionType type = AutoComplete::command);
 
+  //! Add a symbol to the autocompletion list
   void AddSymbol(wxString fun, AutoComplete::autoCompletionType type = AutoComplete::command)
-  { m_autocomplete.AddSymbol(fun, type); }
+  { m_autocomplete->AddSymbol(fun, type); }
+
+  //! Add a xml-encoded list of symbols to the autocompletion list
+  void AddSymbols(wxString xml)
+  { m_autocomplete->AddSymbols(xml); }
 
   void SetActiveCellText(wxString text);
 
   bool InsertText(wxString text);
-  
+
   void OpenNextOrCreateCell();
 
   //! The table of contents pane
@@ -1405,7 +1430,7 @@ public:
   void OpenQuestionCaret(wxString txt = wxT(""));
 
   /*! Returns the cell maxima currently works on. NULL if there isn't such a cell.
-    
+
     \param resortToLast true = if we already have set the cell maxima works on to NULL
     use the last cell maxima was known to work on.
   */
@@ -1427,7 +1452,6 @@ public:
     wxAccStatus GetDescription(int childId, wxString *description);
    private:
     MathCtrl *m_mathCtrl;
-    int m_childCount;
   };
 #endif
 protected:
@@ -1435,7 +1459,8 @@ protected:
   AccessibilityInfo *m_accessibilityInfo;
 #endif
   void UpdateConfigurationClientSize();
-
+  //! Where to start recalculation. NULL = No recalculation needed.
+  GroupCell *m_recalculateStart;
   //! The x position of the mouse pointer
   int m_pointer_x;
   //! The y position of the mouse pointer
