@@ -18,6 +18,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*! \file
   This file defines the class MathParser that reads wxmx data and math from Maxima.
@@ -288,13 +289,7 @@ MathCell *MathParser::ParseEditorTag(wxXmlNode *node)
     {
       if (!text.IsEmpty())
         text += wxT("\n");
-#if wxUSE_UNICODE
       text += line->GetNodeContent();
-#else
-      wxString str = line->GetNodeContent();
-      wxString str1(str.wc_str(wxConvUTF8), *wxConvCurrent);
-      text += str1;
-#endif
     }
     line = line->GetNext();
   } // end while
@@ -354,7 +349,8 @@ MathCell *MathParser::ParseSupTag(wxXmlNode *node)
   wxXmlNode *child = node->GetChildren();
   child = SkipWhitespaceNode(child);
 
-  expt->SetBase(HandleNullPointer(ParseTag(child, false)));
+  MathCell *baseCell;
+    expt->SetBase(baseCell = HandleNullPointer(ParseTag(child, false)));
   child = GetNextTag(child);
 
   MathCell *power = HandleNullPointer(ParseTag(child, false));
@@ -362,7 +358,11 @@ MathCell *MathParser::ParseSupTag(wxXmlNode *node)
   expt->SetPower(power);
   expt->SetType(m_ParserStyle);
   expt->SetStyle(TS_VARIABLE);
+
   ParseCommonAttrs(node, expt);
+  if(node->GetAttribute(wxT("mat"), wxT("false")) == wxT("true"))
+    expt->SetAltCopyText(baseCell->ToString()+wxT("^^")+power->ToString());
+
   return expt;
 }
 
@@ -439,13 +439,7 @@ MathCell *MathParser::ParseText(wxXmlNode *node, int style)
   TextCell *retval = NULL;
   if ((node != NULL) && ((str = node->GetContent()) != wxEmptyString))
   {
-#if !wxUSE_UNICODE
-    wxString str1(str.wc_str(wxConvUTF8), *wxConvCurrent);
-    str = str1;
-#endif
-#if wxUSE_UNICODE
     str.Replace(wxT("-"), wxT("\x2212")); // unicode minus sign
-#endif
 
     wxStringTokenizer lines(str, wxT('\n'));
     while (lines.HasMoreTokens())
@@ -508,10 +502,6 @@ MathCell *MathParser::ParseCharCode(wxXmlNode *node, int style)
     long code;
     if (str.ToLong(&code))
       str = wxString::Format(wxT("%c"), code);
-#if !wxUSE_UNICODE
-    wxString str1(str.wc_str(wxConvUTF8), *wxConvCurrent);
-    str = str1;
-#endif
     cell->SetValue(str);
     cell->SetType(m_ParserStyle);
     cell->SetStyle(style);
@@ -664,6 +654,8 @@ MathCell *MathParser::ParseTableTag(wxXmlNode *node)
     matrix->ColNames(true);
   if (node->GetAttribute(wxT("rownames"), wxT("false")) == wxT("true"))
     matrix->RowNames(true);
+  if (node->GetAttribute(wxT("roundedParens"), wxT("false")) == wxT("true"))
+    matrix->RoundedParens(true);
 
   wxXmlNode *rows = SkipWhitespaceNode(node->GetChildren());
   while (rows)
@@ -855,10 +847,6 @@ MathCell *MathParser::ParseTag(wxXmlNode *node, bool all)
       {
         ImgCell *imageCell;
         wxString filename(node->GetChildren()->GetContent());
-#if !wxUSE_UNICODE
-        wxString filename1(filename.wc_str(wxConvUTF8), *wxConvCurrent);
-        filename = filename1;
-#endif
 
         if (m_fileSystem) // loading from zip
           imageCell = new ImgCell(NULL, m_configuration, m_cellPointers, filename, false, m_fileSystem);
@@ -880,6 +868,10 @@ MathCell *MathParser::ParseTag(wxXmlNode *node, bool all)
             imageCell = new ImgCell(NULL, m_configuration, m_cellPointers, filename, false, NULL);
           }
         }
+        wxString gnuplotSource = node->GetAttribute(wxT("gnuplotsource"), wxEmptyString);
+        wxString gnuplotData = node->GetAttribute(wxT("gnuplotdata"), wxEmptyString);
+        if((imageCell != NULL) && (gnuplotSource != wxEmptyString))
+          imageCell->GnuplotSource(gnuplotSource, gnuplotData, m_fileSystem);
 
         if (node->GetAttribute(wxT("rect"), wxT("true")) == wxT("false"))
           imageCell->DrawRectangle(false);
@@ -921,10 +913,6 @@ MathCell *MathParser::ParseTag(wxXmlNode *node, bool all)
           wxString token = tokens.GetNextToken();
           if (token.Length())
           {
-#if !wxUSE_UNICODE
-            wxString token1(token.wc_str(wxConvUTF8), *wxConvCurrent);
-            token = token1;
-#endif
             images.Add(token);
           }
         }
@@ -1036,23 +1024,14 @@ MathCell *MathParser::ParseLine(wxString s, int style)
       showLength = 50000;    
   }
 
-#if wxUSE_UNICODE
   m_graphRegex.Replace(&s, wxT("\xFFFD"));
-#else
-  m_graphRegex.Replace(&s, wxT("?"));
-#endif
 
   if (((long) s.Length() < showLength) || (showLength == 0))
   {
 
     wxXmlDocument xml;
 
-#if wxUSE_UNICODE
     wxStringInputStream xmlStream(s);
-#else
-    wxString su(s.wc_str(*wxConvCurrent), wxConvUTF8);
-    wxStringInputStream xmlStream(su);
-#endif
 
     xml.Load(xmlStream, wxT("UTF-8"), wxXMLDOC_KEEP_WHITESPACE_NODES);
 
