@@ -18,6 +18,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*! \file
   This file defines the class Structure
@@ -97,10 +98,9 @@ void TableOfContents::UpdateTableOfContents(GroupCell *tree, GroupCell *cursorPo
       cell = dynamic_cast<GroupCell *>(cell->m_next);
     }
 
-    long item = -1;
-    item = m_displayedItems->GetNextItem(-1,
-                                         wxLIST_NEXT_ALL,
-                                         wxLIST_STATE_SELECTED);
+    long item = m_displayedItems->GetNextItem(-1,
+                                              wxLIST_NEXT_ALL,
+                                              wxLIST_STATE_SELECTED);
 
     if ((selection >= 0) && (item != selection))
     {
@@ -116,8 +116,6 @@ void TableOfContents::UpdateTableOfContents(GroupCell *tree, GroupCell *cursorPo
 
 void TableOfContents::UpdateDisplay()
 {
-  wxLogNull disableWarnings;
-
   wxString regex = m_regex->GetValue();
   wxArrayString items;
   wxRegEx matcher;
