@@ -18,6 +18,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*! \file
   This file defines the class TextCell
@@ -27,7 +28,6 @@
  */
 
 #include "TextCell.h"
-#include "Setup.h"
 #include "wx/config.h"
 
 TextCell::TextCell(MathCell *parent, Configuration **config, CellPointers *cellPointers, wxString text) : MathCell(parent, config)
@@ -35,9 +35,6 @@ TextCell::TextCell(MathCell *parent, Configuration **config, CellPointers *cellP
   m_cellPointers = cellPointers;
   m_displayedDigits_old = -1;
   m_height = -1;
-  wxASSERT(m_unescapeRegEx.Compile(wxT("\\\\(.)")));
-  m_labelWidth = -1;
-  m_labelHeight = -1;
   m_realCenter = m_center = -1;
   m_lastCalculationFontSize = -1;
   m_fontSize = -1;
@@ -77,6 +74,8 @@ void TextCell::SetValue(const wxString &text)
   ResetSize();
   m_text.Replace(wxT("\n"), wxEmptyString);
   m_text.Replace(wxT("-->"), wxT("\x2794"));
+  m_text.Replace(wxT(" -->"), wxT("\x2794"));
+  m_text.Replace(wxT(" \x2212\x2192 "), wxT("\x2794"));
   m_text.Replace(wxT("->"), wxT("\x2192"));
   m_text.Replace(wxT("\x2212>"), wxT("\x2192"));
 
@@ -184,12 +183,42 @@ void TextCell::SetValue(const wxString &text)
                         m_displayedText.Right(left);
       m_toolTip = _("The maximum number of displayed digits can be changed in the configuration dialogue");
     }
+    else
+    {
+      if(
+        (m_roundingErrorRegEx1.Matches(m_displayedText)) ||
+        (m_roundingErrorRegEx2.Matches(m_displayedText)) ||
+        (m_roundingErrorRegEx3.Matches(m_displayedText)) ||
+        (m_roundingErrorRegEx4.Matches(m_displayedText))
+        )
+        m_toolTip = _("As calculating 0.1^12 demonstrates maxima by default doesn't tend to "
+                      "hide what looks like being the small error using floating-point "
+                      "numbers introduces.\n"
+                      "If this seems to be the case here the error can be avoided by using "
+                      "exact numbers like 1/10, 1*10^-1 or rat(.1).\n"
+                      "It also can be hidden by setting fpprintprec to an appropriate value. "
+                      "But be aware in this case that even small errors can add up.");
+    }
   }
   else
   {
+    if((text == wxT(" LINE SEARCH FAILED. SEE"))||
+       text.Contains(wxT(" DOCUMENTATION OF ROUTINE MCSRCH")) ||
+       text.Contains(wxT(" POSSIBLE CAUSES: FUNCTION OR GRADIENT ARE INCORRECT")))
+      m_toolTip = _("This message can appear when trying to numerically find an optimum. "
+                    "In this case it might indicate that a starting point lies in a local "
+                    "optimum that fits the data best if one parameter is increased to "
+                    "infinity or decreased to -infinity. It also can indicate that an "
+                    "attempt was made to fit data to an equation that actually matches "
+		    "the data best if one parameter is set to +/- infinity.");
     if(text.StartsWith(wxT("incorrect syntax")) && (text.Contains(wxT("is not an infix operator"))))
       m_toolTip = _("A command or number wasn't preceded by a \":\", a \"$\", a \";\" or a \",\".\n"
         "Most probable cause: A missing comma between two list items.");
+    if(text.StartsWith(wxT("incorrect syntax")) && (text.Contains(wxT("Found LOGICAL expression where ALGEBRAIC expression expected"))))
+      m_toolTip = _("Most probable cause: A dot instead a comma between two list items containing assignments.");
+    if(text.Contains(wxT("Illegal use of delimiter")))
+      m_toolTip = _("Most probable cause: an operator was directly followed by a closing parenthesis.");
+
     if(text.StartsWith(wxT("part: fell off the end.")))
        m_toolTip = _("part() or the [] operator was used in order to extract the nth element "
                      "of something that was less than n elements long.");
@@ -197,10 +226,10 @@ void TextCell::SetValue(const wxString &text)
        m_toolTip = _("The value of few special variables is assigned by Maxima and cannot be changed by the user. Also a few constructs aren't variable names and therefore cannot be written to.");
     if(text.StartsWith(wxT("rat: replaced ")))
       m_toolTip = _("Normally computers use floating-point numbers that can be handled "
-                    "incredibly fast while being accurate to dozends of digits. "
+                    "incredibly fast while being accurate to dozens of digits. "
                     "They will, though, introduce a small error into some common numbers. "
                     "For example 0.1 is represented as 3602879701896397/36028797018963968.\n"
-                    "As mathemathics is based on the fact that numbers that are exactly "
+                    "As mathematics is based on the fact that numbers that are exactly "
                     "equal cancel each other out small errors can quickly add up to big errors "
                     "(see Wilkinson's Polynomials or Rump's Polynomials). Some maxima "
                     "commands therefore use rat() in order to automatically convert floats to "
@@ -212,6 +241,12 @@ void TextCell::SetValue(const wxString &text)
                     "by setting ratprint to false.");
     if(text.StartsWith(wxT("expt: undefined: 0 to a negative exponent.")))
       m_toolTip = _("Division by 0.");
+    if(text.Contains(wxT("arithmetic error DIVISION-BY-ZERO signalled")))
+      m_toolTip = _("Besides a division by 0 the reason for this error message can be a "
+                    "calculation that returns +/-infinity.");
+    if(text.Contains(wxT("isn't in the domain of")))
+      m_toolTip = _("Most probable cause: A function was called with a parameter that causes "
+                    "it to return infinity and/or -infinity.");
     if(text.StartsWith(wxT("Only symbols can be bound")))
       m_toolTip = _("This error message is most probably caused by a try to assign "
                     "a value to a number instead of a variable name.\n"
@@ -265,17 +300,6 @@ MathCell *TextCell::Copy()
   return retval;
 }
 
-wxString TextCell::LabelWidthText()
-{
-  Configuration *configuration = (*m_configuration);
-  wxString result;
-
-  for (int i = 0; i < configuration->GetLabelWidth(); i++)
-    result += wxT("X");
-
-  return result;
-}
-
 void TextCell::RecalculateWidths(int fontsize)
 {
   Configuration *configuration = (*m_configuration);
@@ -349,31 +373,27 @@ void TextCell::RecalculateWidths(int fontsize)
         fontsize1 = 4;
       font.SetPointSize(fontsize1);
       dc->SetFont(font);
-      
-      // Check for output annotations (/R/ for CRE and /T/ for Taylor expressions)
-      if (text.Right(2) != wxT("/ "))
-        dc->GetTextExtent(wxT("(%o") + LabelWidthText() + wxT(")"), &m_width, &m_height);
-      else
-        dc->GetTextExtent(wxT("(%o") + LabelWidthText() + wxT(")/R/"), &m_width, &m_height);
 
+      
+      m_width = Scale_Px(configuration->GetLabelWidth());
       // We will decrease it before use
       m_fontSizeLabel = m_fontSize + 1;
-      wxASSERT_MSG((m_width > 0) || (text == wxEmptyString),
-                   _("The letter \"X\" is of width zero. Installing http://www.math.union.edu/~dpvc/jsmath/download/jsMath-fonts.html and checking \"Use JSmath fonts\" in the configuration dialogue should fix it."));
+      int labelWidth,labelHeight;
       if (m_width < 1) m_width = 10;
-      dc->GetTextExtent(text, &m_labelWidth, &m_labelHeight);
-      wxASSERT_MSG((m_labelWidth > 0) || (m_displayedText == wxEmptyString),
+      dc->GetTextExtent(text, &labelWidth, &labelHeight);
+      wxASSERT_MSG((labelWidth > 0) || (m_displayedText == wxEmptyString),
                    _("Seems like something is broken with the maths font. Installing http://www.math.union.edu/~dpvc/jsmath/download/jsMath-fonts.html and checking \"Use JSmath fonts\" in the configuration dialogue should fix it."));
       font = dc->GetFont();
       do
       {
         font.SetPointSize(Scale_Px(--m_fontSizeLabel));
         dc->SetFont(font);
-        dc->GetTextExtent(text, &m_labelWidth, &m_labelHeight);
-      } while ((m_labelWidth >= m_width) && (m_fontSizeLabel > 2));
+        dc->GetTextExtent(text, &labelWidth, &labelHeight);
+      } while ((labelWidth >= m_width) && (m_fontSizeLabel > 2));
+      m_height = labelHeight;
+      m_center = m_height / 2;
     }
-
-      /// Check if we are using jsMath and have jsMath character
+    // Check if we are using jsMath and have jsMath character
     else if (m_altJs && configuration->CheckTeXFonts())
     {
       dc->GetTextExtent(m_altJsText, &m_width, &m_height);
@@ -399,8 +419,8 @@ void TextCell::RecalculateWidths(int fontsize)
     else
       dc->GetTextExtent(m_displayedText, &m_width, &m_height);
 
-    m_width = m_width + 2 * Scale_Px(MC_TEXT_PADDING);
-    m_height = m_height + 2 * Scale_Px(MC_TEXT_PADDING);
+    m_width = m_width + 2 * MC_TEXT_PADDING;
+    m_height = m_height + 2 * MC_TEXT_PADDING;
 
     /// Hidden cells (multiplication * is not displayed)
     if (m_isHidden)
@@ -409,8 +429,7 @@ void TextCell::RecalculateWidths(int fontsize)
       m_width = m_width / 4;
     }
   }
-  if(m_width < 4) m_width = 4;
-  if(m_height < 4) m_height = 4;
+  if(m_height < Scale_Px(4)) m_height = Scale_Px(4);
   m_realCenter = m_center = m_height / 2;
   ResetData();
 }
@@ -444,40 +463,38 @@ void TextCell::Draw(wxPoint point, int fontsize)
             wxString text = m_userDefinedLabel;
             m_unescapeRegEx.ReplaceAll(&text,wxT("\\1"));
             dc->DrawText(wxT("(") + text + wxT(")"),
-                        point.x + Scale_Px(MC_TEXT_PADDING),
-                        point.y - m_realCenter + (m_height - m_labelHeight) / 2);
+                         point.x + MC_TEXT_PADDING,
+                         point.y - m_realCenter + MC_TEXT_PADDING);
           }
           else
             dc->DrawText(m_displayedText,
-                        point.x + Scale_Px(MC_TEXT_PADDING),
-                        point.y - m_realCenter + (m_height - m_labelHeight) / 2);            
+                         point.x + MC_TEXT_PADDING,
+                         point.y - m_realCenter + MC_TEXT_PADDING);
         }
       }
 
         /// Check if we are using jsMath and have jsMath character
       else if (m_altJs && configuration->CheckTeXFonts())
         dc->DrawText(m_altJsText,
-                    point.x + Scale_Px(MC_TEXT_PADDING),
-                    point.y - m_realCenter + Scale_Px(MC_TEXT_PADDING));
+                    point.x + MC_TEXT_PADDING,
+                    point.y - m_realCenter + MC_TEXT_PADDING);
 
         /// We are using a special symbol
       else if (m_alt)
         dc->DrawText(m_altText,
-                    point.x + Scale_Px(MC_TEXT_PADDING),
-                    point.y - m_realCenter + Scale_Px(MC_TEXT_PADDING));
+                    point.x + MC_TEXT_PADDING,
+                    point.y - m_realCenter + MC_TEXT_PADDING);
 
         /// Change asterisk
       else if (configuration->GetChangeAsterisk() && m_displayedText == wxT("*"))
         dc->DrawText(wxT("\xB7"),
-                    point.x + Scale_Px(MC_TEXT_PADDING),
-                    point.y - m_realCenter + Scale_Px(MC_TEXT_PADDING));
+                    point.x + MC_TEXT_PADDING,
+                    point.y - m_realCenter + MC_TEXT_PADDING);
 
-#if wxUSE_UNICODE
       else if (m_displayedText == wxT("#"))
         dc->DrawText(wxT("\x2260"),
-                    point.x + Scale_Px(MC_TEXT_PADDING),
-                    point.y - m_realCenter + Scale_Px(MC_TEXT_PADDING));
-#endif
+                    point.x + MC_TEXT_PADDING,
+                    point.y - m_realCenter + MC_TEXT_PADDING);
         /// This is the default.
       else
       {
@@ -486,16 +503,16 @@ void TextCell::Draw(wxPoint point, int fontsize)
           case MC_TYPE_TEXT:
             // TODO: Add markdown formatting for bold, italic and underlined here.
             dc->DrawText(m_displayedText,
-                        point.x + Scale_Px(MC_TEXT_PADDING),
-                        point.y - m_realCenter + Scale_Px(MC_TEXT_PADDING));
+                        point.x + MC_TEXT_PADDING,
+                        point.y - m_realCenter + MC_TEXT_PADDING);
             break;
           case MC_TYPE_INPUT:
             // This cell has already been drawn as an EditorCell => we don't repeat this action here.
             break;
           default:
             dc->DrawText(m_displayedText,
-                        point.x + Scale_Px(MC_TEXT_PADDING),
-                        point.y - m_realCenter + Scale_Px(MC_TEXT_PADDING));
+                        point.x + MC_TEXT_PADDING,
+                        point.y - m_realCenter + MC_TEXT_PADDING);
         }
       }
     }
@@ -581,10 +598,8 @@ bool TextCell::IsOperator()
 {
   if (wxString(wxT("+*/-")).Find(m_text) >= 0)
     return true;
-#if wxUSE_UNICODE
   if (m_text == wxT("\x2212"))
     return true;
-#endif
   return false;
 }
 
@@ -598,11 +613,9 @@ wxString TextCell::ToString()
     text = m_text;
     if(((*m_configuration)->UseUserLabels())&&(m_userDefinedLabel != wxEmptyString))
       text = wxT("(") + m_userDefinedLabel + wxT(")");
-#if wxUSE_UNICODE
     text.Replace(wxT("\x2212"), wxT("-")); // unicode minus sign
     text.Replace(wxT("\x2794"), wxT("-->"));
     text.Replace(wxT("\x2192"), wxT("->"));
-#endif
   }
   switch (m_textStyle)
   {
@@ -657,7 +670,9 @@ wxString TextCell::ToString()
         break;
       }
   }
-
+  if((m_next != NULL) && (m_next->BreakLine()))
+    text += "\n";
+  
   return text;
 }
 
@@ -724,7 +739,6 @@ wxString TextCell::ToTeX()
 
   text.Replace(wxT("<"), mathModeStart + wxT("<") + mathModeEnd);
   text.Replace(wxT(">"), mathModeStart + wxT(">") + mathModeEnd);
-#if wxUSE_UNICODE
   text.Replace(wxT("\x2212"), wxT("-")); // unicode minus sign
   text.Replace(L"\x00B1", mathModeStart + wxT("\\pm") + mathModeEnd);
   text.Replace(L"\x03B1", mathModeStart + wxT("\\alpha") + mathModeEnd);
@@ -829,8 +843,6 @@ wxString TextCell::ToTeX()
   text.Replace(wxT("\x219D"), mathModeStart + wxT("\\leadsto") + mathModeEnd);
   text.Replace(wxT("\x2192"), mathModeStart + wxT("\\rightarrow") + mathModeEnd);
   text.Replace(wxT("\x2794"), mathModeStart + wxT("\\longrightarrow") + mathModeEnd);
-#endif
-
 
   // m_IsHidden is set for multiplication signs and parenthesis that
   // don't need to be shown
@@ -1247,6 +1259,8 @@ wxString TextCell::ToRTF()
     text = wxT("(") + m_userDefinedLabel + wxT(")");
   
   text.Replace(wxT("-->"), wxT("\x2192"));
+  // Needed for the output of let(a/b,a+1);
+  text.Replace(wxT(" --> "), wxT("\x2192"));
   if ((GetStyle() == TS_LABEL) || (GetStyle() == TS_USERLABEL))
   {
     retval += wxString::Format(wxT("\\cf%i{"), (int) GetStyle());
@@ -1330,7 +1344,7 @@ bool TextCell::IsShortNum()
 void TextCell::SetAltText()
 {
   m_altJs = m_alt = false;
-  if (GetStyle() == TS_DEFAULT)
+  if ((GetStyle() == TS_DEFAULT) && m_text.StartsWith("\""))
     return;
 
   /// Greek characters are defined in jsMath, Windows and Unicode
@@ -1340,14 +1354,8 @@ void TextCell::SetAltText()
     m_altJsText = GetGreekStringTeX();
     m_texFontname = CMMI10;
 
-#if wxUSE_UNICODE
     m_alt = true;
     m_altText = GetGreekStringUnicode();
-#elif defined __WXMSW__
-    m_alt = true;
-    m_altText = GetGreekStringSymbol();
-    m_fontname = wxT("Symbol");
-#endif
   }
 
     /// Check for other symbols
@@ -1364,23 +1372,20 @@ void TextCell::SetAltText()
         m_texFontname = CMSY10;
       m_altJs = true;
     }
-#if wxUSE_UNICODE
     m_altText = GetSymbolUnicode((*m_configuration)->CheckKeepPercent());
     if (m_altText != wxEmptyString)
       m_alt = true;
-#elif defined __WXMSW__
-    m_altText = GetSymbolSymbol(configuration->CheckKeepPercent());
-    if (m_altText != wxEmptyString)
-    {
-      m_alt = true;
-      m_fontname = wxT("Symbol");
-    }
-#endif
+// #if defined __WXMSW__
+//     m_altText = GetSymbolSymbol(configuration->CheckKeepPercent());
+//     if (m_altText != wxEmptyString)
+//     {
+//       m_alt = true;
+//       m_fontname = wxT("Symbol");
+//     }
+// #endif
   }
 }
 
-#if wxUSE_UNICODE
-
 wxString TextCell::GetGreekStringUnicode()
 {
   wxString txt(m_text);
@@ -1502,7 +1507,7 @@ wxString TextCell::GetSymbolUnicode(bool keepPercent)
     return wxT("\x2264");
   else if (m_text == wxT(">="))
     return wxT("\x2265");
-#ifndef __WXMSW__
+  #ifndef __WXMSW__
   else if (m_text == wxT(" and "))
     return wxT(" \x22C0 ");
   else if (m_text == wxT(" or "))
@@ -1519,11 +1524,16 @@ wxString TextCell::GetSymbolUnicode(bool keepPercent)
     return wxT(" \x21D4 ");
   else if (m_text == wxT("not"))
     return wxT("\x00AC");
+  #endif
   else if (m_text == wxT("->"))
     return wxT("\x2192");
   else if (m_text == wxT("-->"))
     return wxT("\x2794");
-#endif
+  // The next two ones are needed for the output of let(a/b,a+1);
+  else if (m_text == wxT(" --> "))
+    return wxT("\x2794");
+  else if (m_text == wxT(" \x2212\x2192 "))
+    return wxT("\x2794");
   /*
    else if (GetStyle() == TS_SPECIAL_CONSTANT && m_text == wxT("d"))
      return wxT("\x2202");
@@ -1542,165 +1552,6 @@ wxString TextCell::GetSymbolUnicode(bool keepPercent)
   return wxEmptyString;
 }
 
-#elif defined __WXMSW__
-
-wxString TextCell::GetGreekStringSymbol()
-{
-  if (m_text == wxT("gamma"))
-    return wxT("\x47");
-  else if (m_text == wxT("zeta"))
-    return wxT("\x7A");
-  else if (m_text == wxT("psi"))
-    return wxT("\x59");
-
-  wxString txt(m_text);
-  if (txt[0] != '%')
-    txt = wxT("%") + txt;
-
-  if (txt == wxT("%alpha"))
-    return wxT("\x61");
-  else if (txt == wxT("%beta"))
-    return wxT("\x62");
-  else if (txt == wxT("%gamma"))
-    return wxT("\x67");
-  else if (txt == wxT("%delta"))
-    return wxT("\x64");
-  else if (txt == wxT("%epsilon"))
-    return wxT("\x65");
-  else if (txt == wxT("%zeta"))
-    return wxT("\x7A");
-  else if (txt == wxT("%eta"))
-    return wxT("\x68");
-  else if (txt == wxT("%theta"))
-    return wxT("\x71");
-  else if (txt == wxT("%iota"))
-    return wxT("\x69");
-  else if (txt == wxT("%kappa"))
-    return wxT("\x6B");
-  else if (txt == wxT("%lambda"))
-    return wxT("\x6C");
-  else if (txt == wxT("%mu"))
-    return wxT("\x6D");
-  else if (txt == wxT("%nu"))
-    return wxT("\x6E");
-  else if (txt == wxT("%xi"))
-    return wxT("\x78");
-  else if (txt == wxT("%omicron"))
-    return wxT("\x6F");
-  else if (txt == wxT("%pi"))
-    return wxT("\x70");
-  else if (txt == wxT("%rho"))
-    return wxT("\x72");
-  else if (txt == wxT("%sigma"))
-    return wxT("\x73");
-  else if (txt == wxT("%tau"))
-    return wxT("\x74");
-  else if (txt == wxT("%upsilon"))
-    return wxT("\x75");
-  else if (txt == wxT("%phi"))
-    return wxT("\x66");
-  else if (txt == wxT("%chi"))
-    return wxT("\x63");
-  else if (txt == wxT("%psi"))
-    return wxT("\x79");
-  else if (txt == wxT("%omega"))
-    return wxT("\x77");
-  else if (txt == wxT("%Alpha"))
-    return wxT("\x41");
-  else if (txt == wxT("%Beta"))
-    return wxT("\x42");
-  else if (txt == wxT("%Gamma"))
-    return wxT("\x47");
-  else if (txt == wxT("%Delta"))
-    return wxT("\x44");
-  else if (txt == wxT("%Epsilon"))
-    return wxT("\x45");
-  else if (txt == wxT("%Zeta"))
-    return wxT("\x5A");
-  else if (txt == wxT("%Eta"))
-    return wxT("\x48");
-  else if (txt == wxT("%Theta"))
-    return wxT("\x51");
-  else if (txt == wxT("%Iota"))
-    return wxT("\x49");
-  else if (txt == wxT("%Kappa"))
-    return wxT("\x4B");
-  else if (txt == wxT("%Lambda"))
-    return wxT("\x4C");
-  else if (txt == wxT("%Mu"))
-    return wxT("\x4D");
-  else if (txt == wxT("%Nu"))
-    return wxT("\x4E");
-  else if (txt == wxT("%Xi"))
-    return wxT("\x58");
-  else if (txt == wxT("%Omicron"))
-    return wxT("\x4F");
-  else if (txt == wxT("%Pi"))
-    return wxT("\x50");
-  else if (txt == wxT("%Rho"))
-    return wxT("\x52");
-  else if (txt == wxT("%Sigma"))
-    return wxT("\x53");
-  else if (txt == wxT("%Tau"))
-    return wxT("\x54");
-  else if (txt == wxT("%Upsilon"))
-    return wxT("\x55");
-  else if (txt == wxT("%Phi"))
-    return wxT("\x46");
-  else if (txt == wxT("%Chi"))
-    return wxT("\x43");
-  else if (txt == wxT("%Psi"))
-    return wxT("\x59");
-  else if (txt == wxT("%Omega"))
-    return wxT("\x57");
-
-  return wxEmptyString;
-}
-
-wxString TextCell::GetSymbolSymbol(bool keepPercent)
-{
-  if (m_text == wxT("inf"))
-    return "\xA5";
-  else if (m_text == wxT("%pi"))
-    return "\x70";
-  else if (m_text == wxT("->"))
-    return "\xAE";
-  else if (m_text == wxT(">="))
-    return "\xB3";
-  else if (m_text == wxT("<="))
-    return "\xA3";
-  else if (m_text == wxT(" and "))
-    return "\xD9";
-  else if (m_text == wxT(" or "))
-    return "\xDA";
-  else if (m_text == wxT("not"))
-    return "\xD8";
-  else if (m_text == wxT(" nand "))
-    return "\xAD";
-  else if (m_text == wxT(" nor "))
-    return "\xAF";
-  else if (m_text == wxT(" implies "))
-    return "\xDE";
-  else if (m_text == wxT(" equiv "))
-    return "\xDB";
-  else if (m_text == wxT(" xor "))
-    return "\xC5";
-  else if (m_text == wxT("~>"))
-    return "\x219D";
-
-      
-  if (!keepPercent) {
-    if (m_text == wxT("%e"))
-      return wxT("e");
-    else if (m_text == wxT("%i"))
-      return wxT("i");
-  }
-
-  return wxEmptyString;
-}
-
-#endif
-
 wxString TextCell::GetGreekStringTeX()
 {
   if (m_text == wxT("gamma"))
@@ -1851,3 +1702,11 @@ wxString TextCell::GetSymbolTeX()
 
   return wxEmptyString;
 }
+
+
+// RegExes all TextCells share.
+wxRegEx TextCell::m_unescapeRegEx(wxT("\\\\(.)"));
+wxRegEx TextCell::m_roundingErrorRegEx1(wxT("\\.000000000000[0-9]+$"));
+wxRegEx TextCell::m_roundingErrorRegEx2(wxT("\\.999999999999[0-9]+$"));
+wxRegEx TextCell::m_roundingErrorRegEx3(wxT("\\.000000000000[0-9]+e"));
+wxRegEx TextCell::m_roundingErrorRegEx4(wxT("\\.999999999999[0-9]+e"));
