@@ -20,6 +20,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*! \file
   This file defines the class ToolBar that represents wxMaxima's main tool bar.
@@ -29,7 +30,6 @@
 #include "../art/toolbar/images.h"
 #include <wx/mstream.h>
 #include <wx/wfstream.h>
-#include "Dirstructure.h"
 #include "GroupCell.h"
 #include <wx/artprov.h>
 #include <wx/filename.h>
@@ -41,11 +41,11 @@
 #define MAX(a, b) ((a)>(b) ? (a) : (b))
 #define MIN(a, b) ((a)>(b) ? (b) : (a))
 
-wxImage ToolBar::GetImage(wxString name,
+wxBitmap ToolBar::GetImage(wxString name,
                           unsigned char *data_128, size_t len_128,
                           unsigned char *data_192, size_t len_192)
 {
-  double targetSize = wxGetDisplayPPI().x * TOOLBAR_ICON_SCALE;
+  double targetSize = wxGetDisplayPPI().x * TOOLBAR_ICON_SCALE * GetContentScaleFactor();
   int prescale;
 
   int sizeA = 128 << 4;
@@ -92,14 +92,27 @@ wxImage ToolBar::GetImage(wxString name,
     img = wxImage(invalidImage_xpm);
   }
 
+#if defined __WXMSW__
+#if wxCHECK_VERSION(3, 1, 1)
+  // MSW is notorious for having problems with transparent black pixels.
+  // Let's see if we can avoid these problems by converting the alpha
+  // channel to a mask even if that means we cannot antialias a transparent 
+  // and a colored pixel to a half-transparent one any more.
+  img.ConvertAlphaToMask();
+#endif
+#endif
+  
   img.Rescale(targetSize, targetSize, wxIMAGE_QUALITY_HIGH);
-  return img;
+#if defined __WXMAC__
+  return wxBitmap(img,wxBITMAP_SCREEN_DEPTH,GetContentScaleFactor());
+#else
+  return wxBitmap(img,wxBITMAP_SCREEN_DEPTH);
+#endif
 }
 
 ToolBar::~ToolBar()
 {
   m_plotSlider = NULL;
-  m_toolBar->Destroy();
 }
 
 void ToolBar::UpdateSlider(SlideShow *cell)
@@ -122,7 +135,9 @@ void ToolBar::UpdateSlider(SlideShow *cell)
   }
 }
 
-ToolBar::ToolBar(wxToolBar *tbar)
+ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition, wxDefaultSize,
+                                                  wxAUI_TB_OVERFLOW | wxAUI_TB_PLAIN_BACKGROUND |
+                                                  wxAUI_TB_HORIZONTAL)
 {
   m_defaultCellStyle = GC_TYPE_CODE;
   m_canCopy_old = true;
@@ -131,93 +146,91 @@ ToolBar::ToolBar(wxToolBar *tbar)
   m_canPrint_old = true;
   m_canEvalTillHere_old = true;
 
-  m_toolBar = tbar;
   m_needsInformation = false;
   m_AnimationStartStopState = Inactive;
 
-  m_toolBar->SetToolBitmapSize(wxSize(24, 24));
-
-#if defined __WXMSW__
-  m_toolBar->AddTool(tb_new, _("New"),
+  SetToolBitmapSize(wxSize(24, 24));
+  SetGripperVisible(false);
+  
+  AddTool(tb_new, _("New"),
                      GetImage(wxT("gtk-new"),
                               gtk_new_128_png,gtk_new_128_png_len,
                               gtk_new_192_png,gtk_new_192_png_len
                        ),
                      _("New document"));
-#endif
-  m_toolBar->AddTool(tb_open, _("Open"),
+  AddTool(tb_open, _("Open"),
                      GetImage(wxT("gtk-open"),
                               gtk_open_128_png,gtk_open_128_png_len,
                               gtk_open_192_png,gtk_open_192_png_len
                        ),
                      _("Open document"));
-  m_toolBar->AddTool(tb_save, _("Save"),
+  AddTool(tb_save, _("Save"),
                      GetImage(wxT("gtk-save"),
                               gtk_save_128_png,gtk_save_128_png_len,
                               gtk_save_192_png,gtk_save_192_png_len
                        ),
                      _("Save document"));
 #ifndef __WXMAC__
-  m_toolBar->AddSeparator();
+  AddSeparator();
 #endif
-  m_toolBar->AddTool(tb_print, _("Print"),
+  AddTool(tb_print, _("Print"),
                      GetImage(wxT("gtk-print"),
                               gtk_print_128_png,gtk_print_128_png_len,
                               gtk_print_192_png,gtk_print_192_png_len
                        ),
                      _("Print document"));
-  m_toolBar->AddTool(tb_pref, _("Options"),
+  AddTool(tb_pref, _("Options"),
                      GetImage(wxT("gtk-preferences"),
                               gtk_preferences_128_png,gtk_preferences_128_png_len,
                               gtk_preferences_192_png,gtk_preferences_192_png_len
                        ),
                      _("Configure wxMaxima"));
 #ifndef __WXMAC__
-  m_toolBar->AddSeparator();
+  AddSeparator();
 #endif
-  m_toolBar->AddTool(tb_cut, _("Cut"),
+  AddTool(tb_cut, _("Cut"),
                      GetImage(wxT("gtk-cut"),
                               gtk_cut_128_png,gtk_cut_128_png_len,
                               gtk_cut_192_png,gtk_cut_192_png_len
                        ),
                      _("Cut selection"));
-  m_toolBar->AddTool(tb_copy, _("Copy"),
+  AddTool(tb_copy, _("Copy"),
                      GetImage(wxT("gtk-copy"),
                               gtk_copy_128_png,gtk_copy_128_png_len,
                               gtk_copy_192_png,gtk_copy_192_png_len
                        ),
                      _("Copy selection"));
-  m_toolBar->AddTool(tb_paste, _("Paste"),
+  AddTool(tb_paste, _("Paste"),
                      GetImage(wxT("gtk-paste"),
                               gtk_paste_128_png,gtk_paste_128_png_len,
                               gtk_paste_192_png,gtk_paste_192_png_len
                        ),
                      _("Paste from clipboard"));
-  m_toolBar->AddTool(tb_select_all, _("Select all"),
+  AddTool(tb_select_all, _("Select all"),
                      GetImage(wxT("gtk-select-all"),
                               gtk_select_all_128_png,gtk_select_all_128_png_len,
                               gtk_select_all_192_png,gtk_select_all_192_png_len
                        ),
                      _("Select all"));
 #ifndef __WXMAC__
-  m_toolBar->AddSeparator();
+  AddSeparator();
 #endif
-  m_toolBar->AddTool(tb_find, _("Find"),
+  AddTool(tb_find, _("Find"),
                      GetImage(wxT("gtk-find"),
                               gtk_find_128_png,gtk_find_128_png_len,
                               gtk_find_192_png,gtk_find_192_png_len
                        ),
                      _("Find and replace"));
 #ifndef __WXMAC__
-  m_toolBar->AddSeparator();
+  AddSeparator();
 #endif
-  m_toolBar->AddTool(menu_restart_id, _("Restart maxima"),
+  AddTool(menu_restart_id, _("Restart maxima"),
                      GetImage(wxT("view-refresh"),
                               view_refresh_128_png,view_refresh_128_png_len,
                               view_refresh_192_png,view_refresh_192_png_len
                        ),
                      _("Completely stop maxima and restart it"));
-  m_toolBar->AddTool(tb_interrupt, _("Interrupt"),
+  AddTool(tb_interrupt, _("Interrupt"),
                      GetImage(wxT("gtk-stop"),
                               gtk_stop_128_png,gtk_stop_128_png_len,
                               gtk_stop_192_png,gtk_stop_192_png_len
@@ -231,19 +244,35 @@ ToolBar::ToolBar(wxToolBar *tbar)
                               software_update_urgent_128_png,software_update_urgent_128_png_len,
                               software_update_urgent_192_png,software_update_urgent_192_png_len
     );
-  m_toolBar->AddTool(tb_follow, _("Follow"), m_followIcon,
+  AddTool(tb_follow, _("Follow"), m_followIcon,
                      _("Return to the cell that is currently being evaluated"));
-  m_toolBar->EnableTool(tb_follow, false);
+  EnableTool(tb_follow, false);
 
-  m_toolBar->AddTool(tb_evaltillhere, _("Evaluate to point"),
+  AddTool(tb_evaltillhere, _("Evaluate to point"),
                      GetImage(wxT("go-bottom"),
                               go_bottom_128_png,go_bottom_128_png_len,
                               go_bottom_192_png,go_bottom_192_png_len
                        ),
                      _("Evaluate the file from its beginning to the cell above the cursor"));
 
+    AddTool(tb_evaluate_rest, _("Evaluate the rest"),
+                     GetImage(wxT("go-last"),
+                              go_last_128_png,go_last_128_png_len,
+                              go_last_192_png,go_last_192_png_len
+                       ),
+                     _("Evaluate the file from the cursor to its end"));
+
 #ifndef __WXMAC__
-  m_toolBar->AddSeparator();
+    AddSeparator();
+#endif
+    AddTool(tb_hideCode, _("Hide Code"),
+            GetImage(wxT("weather-few-clouds"),
+                     weather_few_clouds_128_png,weather_few_clouds_128_png_len,
+                     weather_few_clouds_192_png,weather_few_clouds_192_png_len
+              ),
+            _("Toggle the visibility of code cells"));
+#ifndef __WXMAC__
+  AddSeparator();
 #endif
   wxArrayString textStyle;
   textStyle.Add(_("Maths"));
@@ -252,7 +281,7 @@ ToolBar::ToolBar(wxToolBar *tbar)
   textStyle.Add(_("Section"));
   textStyle.Add(_("Subsection"));
   textStyle.Add(_("Subsubsection"));
-  m_textStyle = new wxChoice(m_toolBar, tb_changeStyle, wxDefaultPosition, wxDefaultSize, textStyle);
+  m_textStyle = new wxChoice(this, tb_changeStyle, wxDefaultPosition, wxDefaultSize, textStyle);
   m_textStyle->SetToolTip(_("For faster creation of cells the following shortcuts exist:\n\n"
                             "   Ctrl+0: Math cell\n"
                             "   Ctrl+1: Text cell\n"
@@ -260,24 +289,14 @@ ToolBar::ToolBar(wxToolBar *tbar)
                             "   Ctrl+3: Section cell\n"
                             "   Ctrl+4: Subsection cell\n"
                             "   Ctrl+5: Sub-Subsection cell"));
-  m_toolBar->AddControl(m_textStyle);
-#ifndef __WXMAC__
-  m_toolBar->AddSeparator();
-#endif
+  AddControl(m_textStyle);
   
   // Seems like on MSW changing the image of this button has strange side-effects
   // so we combine both images into one for this OS.
-#if defined __WXMSW__
-  m_PlayButton = GetImage(wxT("media-playback-startstop"),
-                              media_playback_startstop_128_png,media_playback_startstop_128_png_len,
-                              media_playback_startstop_192_png,media_playback_startstop_192_png_len
-    );
-#else
   m_PlayButton = GetImage(wxT("media-playback-start"),
                               media_playback_start_128_png,media_playback_start_128_png_len,
                               media_playback_start_192_png,media_playback_start_192_png_len
     );
-#endif
   m_StopButton = GetImage(wxT("media-playback-stop"),
                               media_playback_stop_128_png,media_playback_stop_128_png_len,
                               media_playback_stop_192_png,media_playback_stop_192_png_len
@@ -286,16 +305,17 @@ ToolBar::ToolBar(wxToolBar *tbar)
   // It felt like a good idea to combine the play and the stop button.
   // On windows changing a button seems to somehow stop the animation, though, so
   // this OS requires the buttons to be separate.
-  m_toolBar->AddTool(tb_animation_startStop, _("Start or Stop animation"),
+  AddTool(tb_animation_startStop, _("Start or Stop animation"),
                      m_PlayButton,
                      _("Start or stop the currently selected animation that has been created with the with_slider class of commands"));
-  m_toolBar->EnableTool(tb_animation_startStop, false);
+  EnableTool(tb_animation_startStop, false);
+
   int sliderWidth = wxGetDisplayPPI().x * 200 / 72;
   int width, height;
   wxDisplaySize(&width, &height);
   if (width < 800)
     sliderWidth = MIN(sliderWidth, 100);
-  m_plotSlider = new wxSlider(m_toolBar, plot_slider_id, 0, 0, 10,
+  m_plotSlider = new wxSlider(this, plot_slider_id, 0, 0, 10,
                               wxDefaultPosition, wxSize(sliderWidth, -1),
                               wxSL_HORIZONTAL | !wxSL_AUTOTICKS);
   m_plotSlider->SetToolTip(
@@ -303,24 +323,20 @@ ToolBar::ToolBar(wxToolBar *tbar)
   m_plotSlider->Enable(false);
   m_slideShowMaxIndex = -1;
   m_slideShowDisplayedIndex = -1;
-  m_toolBar->AddControl(m_plotSlider);
-#ifndef __WXMAC__
-  m_toolBar->AddSeparator();
-#endif
-  m_toolBar->AddTool(tb_hideCode, _("Hide Code"),
-                     GetImage(wxT("weather-few-clouds"),
-                              weather_few_clouds_128_png,weather_few_clouds_128_png_len,
-                              weather_few_clouds_192_png,weather_few_clouds_192_png_len
-                       ),
-                     _("Toggle the visibility of code cells"));
-  m_toolBar->AddStretchableSpace();
-  m_toolBar->AddTool(tb_help, _("Help"),
+  AddControl(m_plotSlider);
+  AddStretchSpacer();
+  AddTool(tb_help, _("Help"),
                      GetImage(wxT("gtk-help"),
                               gtk_help_128_png,gtk_help_128_png_len,
                               gtk_help_192_png,gtk_help_192_png_len
                        ),
                      _("Show Maxima help"));
-  m_toolBar->Realize();
+  Connect(wxEVT_SIZE,
+          wxSizeEventHandler(ToolBar::OnSize),
+          NULL, this);
+  Realize();
+  SetInitialSize(wxSize(GetBestSize().x,100000));
+  Realize();
 }
 
 void ToolBar::SetDefaultCellStyle()
@@ -430,24 +446,20 @@ void ToolBar::AnimationButtonState(AnimationStartStopState state)
         m_plotSlider->Enable(true);
         if (m_AnimationStartStopState != Running)
         {
-#ifndef __WXMSW__
-          m_toolBar->SetToolNormalBitmap(tb_animation_startStop, m_StopButton);
-#endif
+          SetToolBitmap(tb_animation_startStop, m_StopButton);
         }
-        m_toolBar->EnableTool(tb_animation_startStop, true);
+        EnableTool(tb_animation_startStop, true);
         break;
       case Stopped:
         if (m_AnimationStartStopState == Running)
         {
-#ifndef __WXMSW__
-          m_toolBar->SetToolNormalBitmap(tb_animation_startStop, m_PlayButton);
-#endif
+          SetToolBitmap(tb_animation_startStop, m_PlayButton);
         }
-        m_toolBar->EnableTool(tb_animation_startStop, true);
+        EnableTool(tb_animation_startStop, true);
         m_plotSlider->Enable(true);
         break;
       case Inactive:
-        m_toolBar->EnableTool(tb_animation_startStop, false);
+        EnableTool(tb_animation_startStop, false);
         m_plotSlider->Enable(false);
         m_plotSlider->SetToolTip(
                 _("After clicking on animations created with with_slider_draw() or similar this slider allows to change the current frame."));
@@ -456,12 +468,18 @@ void ToolBar::AnimationButtonState(AnimationStartStopState state)
 
         if (m_AnimationStartStopState == Running)
         {
-#ifndef __WXMSW__
-          m_toolBar->SetToolNormalBitmap(tb_animation_startStop, m_PlayButton);
-#endif
+          SetToolBitmap(tb_animation_startStop, m_PlayButton);
         }
         break;
     }
     m_AnimationStartStopState = state;
   }
 }
+
+void ToolBar::OnSize(wxSizeEvent &event)
+{
+  wxLogMessage("ToolBar resizes");
+  Refresh();
+  event.Skip();
+}
+
