@@ -20,6 +20,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*! \file
   This file declares the class ToolBar that represents wxMaxima's main tool bar.
@@ -33,9 +34,10 @@
 #ifndef _WXMAXIMA_TOOLBAR_H
 #define _WXMAXIMA_TOOLBAR_H
 
-class ToolBar
+class ToolBar : public wxAuiToolBar
 {
 public:
+  ToolBar(wxWindow *parent);
   /*! All states the "start/stop animation" toolbar button can be in
    */
   enum AnimationStartStopState
@@ -45,11 +47,9 @@ public:
     Inactive //!< No animation is currently running
   };
   
-  wxImage GetImage(wxString name,
-                   unsigned char *data_128, size_t len_128,
-                   unsigned char *data_192, size_t len_192);
-
-  ToolBar(wxToolBar *tbar);
+  wxBitmap GetImage(wxString name,
+                    unsigned char *data_128, size_t len_128,
+                    unsigned char *data_192, size_t len_192);
 
   virtual ~ToolBar();
 
@@ -58,7 +58,7 @@ public:
   {
     if (!m_needsInformation)
     {
-      m_toolBar->SetToolNormalBitmap(tb_follow, m_needsInformationIcon);
+      SetToolBitmap(tb_follow, m_needsInformationIcon);
       m_needsInformation = true;
     }
   }
@@ -68,21 +68,11 @@ public:
   {
     if (m_needsInformation)
     {
-      m_toolBar->SetToolNormalBitmap(tb_follow, m_followIcon);
+      SetToolBitmap(tb_follow, m_followIcon);
       m_needsInformation = false;
     }
   }
 
-  void EnableTool(int id, bool enable)
-  {
-    m_toolBar->EnableTool(id, enable);
-  }
-
-  wxToolBar *GetToolBar()
-  {
-    return m_toolBar;
-  }
-
   void AnimationButtonState(AnimationStartStopState state);
 
   /*! A list of all events the Toolbar can receive
@@ -116,17 +106,10 @@ public:
   //! The slider for animations
   wxSlider *m_plotSlider;
 
-#if defined __WXGTK__
   wxBitmap  m_followIcon;
   wxBitmap  m_needsInformationIcon;
   wxBitmap  m_PlayButton;
   wxBitmap  m_StopButton;
-#else
-  wxImage m_followIcon;
-  wxImage m_needsInformationIcon;
-  wxImage m_PlayButton;
-  wxImage m_StopButton;
-#endif
 
   void CanCopy(bool value)
   {
@@ -169,6 +152,7 @@ public:
     if (value != m_canEvalTillHere_old)
     {
       EnableTool(tb_evaltillhere, value);
+      EnableTool(tb_evaluate_rest, value);
       m_canEvalTillHere_old = value;
     }
   }
@@ -189,7 +173,9 @@ public:
     }
   //! The current style is the new style for new cells
   void SetDefaultCellStyle();
-  
+
+protected:
+    void OnSize(wxSizeEvent &event);
 private:
   //! The default style for new cells.
   int m_defaultCellStyle;
@@ -204,11 +190,9 @@ private:
   bool m_canSave_old;
   bool m_canPrint_old;
   bool m_canEvalTillHere_old;
-  wxToolBar *m_toolBar;
   AnimationStartStopState m_AnimationStartStopState;
   //! True if we show the "needs information" button.
   bool m_needsInformation;
-
 };
 
 #endif
