@@ -18,6 +18,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*!\file
   The file that defines the starting point of the program
@@ -32,11 +33,12 @@
 
 #include <wx/cmdline.h>
 #include <wx/fileconf.h>
+#include <wx/sysopt.h>
 #include "Dirstructure.h"
 #include <iostream>
 
 #include "wxMaxima.h"
-#include "Setup.h"
+#include "Version.h"
 
 // On wxGTK2 we support printing only if wxWidgets is compiled with gnome_print.
 // We have to force gnome_print support to be linked in static builds of wxMaxima.
@@ -59,6 +61,37 @@ void MyApp::Cleanup_Static()
 
 bool MyApp::OnInit()
 {
+  // MSW: Perhaps that is faster.
+  wxSystemOptions::SetOption("msw.display.directdraw","1");
+  // No spell checking in our dialog's input portions on the mac.
+  wxSystemOptions::SetOption("mac.textcontrol-use-spell-checker","0");
+  
+  // Migrate an eventual old config file to the location XDG wants it to be.
+  #ifndef __WXMSW__
+  #if wxCHECK_VERSION(3, 1, 1)
+  wxStandardPaths::Get().SetFileLayout(wxStandardPaths::FileLayout_Classic);
+  wxString configFileOld = wxStandardPaths::Get().GetUserConfigDir() + wxT("/") +
+    wxStandardPaths::Get().MakeConfigFileName(
+      wxString(wxT("wxMaxima")),
+      wxStandardPaths::ConfigFileConv_Dot);
+  wxStandardPaths::Get().SetFileLayout(wxStandardPaths::FileLayout_XDG);
+  wxString configFileXDG = wxStandardPaths::Get().GetUserConfigDir() + wxT("/") +
+    wxStandardPaths::Get().MakeConfigFileName(
+      wxString(wxT("wxMaxima")),
+      wxStandardPaths::ConfigFileConv_Ext);
+
+  if(!wxFileExists(configFileXDG))
+  {
+    wxFileName xdgDir(configFileXDG);
+    wxString dirName(xdgDir.GetPath());
+    if(!wxDirExists(dirName))
+      wxMkDir(dirName,0x700);
+    if(wxFileExists(configFileOld))
+      wxCopyFile(configFileOld,configFileXDG);
+  }
+  #endif
+  #endif
+  
   m_frame = NULL;
 //  atexit(Cleanup_Static);
   int lang = wxLANGUAGE_UNKNOWN;
@@ -96,15 +129,13 @@ bool MyApp::OnInit()
   }
   else
     wxConfig::Set(new wxConfig(wxT("wxMaxima")));
-
+  
   wxImage::AddHandler(new wxPNGHandler);
   wxImage::AddHandler(new wxXPMHandler);
   wxImage::AddHandler(new wxJPEGHandler);
 
   wxFileSystem::AddHandler(new wxZipFSHandler);
 
-  Dirstructure dirstructure;
-
   wxConfigBase *config = wxConfig::Get();
   lang = wxLocale::GetSystemLanguage();
   config->Read(wxT("language"), &lang);
@@ -114,6 +145,8 @@ bool MyApp::OnInit()
   else
     m_locale.Init(wxLANGUAGE_ENGLISH);
 
+  Dirstructure dirstruct;
+
 #if defined (__WXMSW__)
   wxSetEnv(wxT("LANG"), m_locale.GetName());
   if (!wxGetEnv(wxT("BUILD_DIR"), NULL))
@@ -122,8 +155,7 @@ bool MyApp::OnInit()
     if(dir != wxEmptyString)
       wxSetWorkingDirectory(wxPathOnly(wxStandardPaths::Get().GetExecutablePath()));
   }
-  
-  Dirstructure dirstruct;
+
   wxString fontPrefix = dirstruct.FontDir() + wxT("/");
   
   /* Add private jsMath fonts, if they exist */ 
@@ -146,8 +178,8 @@ bool MyApp::OnInit()
   if (wxFileExists(fontPrefix + wxT(LIBERTINE9))) AddFontResource(fontPrefix + wxT(LIBERTINE9));
 #endif
 
-  m_locale.AddCatalogLookupPathPrefix(dirstructure.LocaleDir());
-  m_locale.AddCatalogLookupPathPrefix(dirstructure.LocaleDir()+wxT("/wxwin"));
+  m_locale.AddCatalogLookupPathPrefix(dirstruct.LocaleDir());
+  m_locale.AddCatalogLookupPathPrefix(dirstruct.LocaleDir()+wxT("/wxwin"));
   m_locale.AddCatalogLookupPathPrefix(wxT("/usr/share/locale"));
   m_locale.AddCatalogLookupPathPrefix(wxT("/usr/local/share/locale"));
   m_locale.AddCatalog(wxT("wxMaxima"));
@@ -287,7 +319,7 @@ void MyApp::NewWindow(wxString file, bool evalOnStartup, bool exitAfterEval)
   if (m == 1)
     m_frame->Maximize(true);
 
-  if (file.Length() > 0 && wxFileExists(file))
+  if (file.Length() > 0)
   {
     m_frame->SetOpenFile(file);
   }
@@ -319,6 +351,7 @@ void MyApp::OnFileMenu(wxCommandEvent &ev)
 #if defined __WXMAC__
       NewWindow();
 #else
+//      NewWindow();
       wxExecute(wxT("\"")+wxStandardPaths::Get().GetExecutablePath()+wxT("\""));
 #endif
       break;
