@@ -18,6 +18,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*!\file
   This file declares the class wxMaxima that contains most of the program's logic. 
@@ -132,12 +133,6 @@ public:
   //! A timer that polls for output from the maxima process.
   wxTimer m_maximaStdoutPollTimer;
 
-  /*! The interval between auto-saves (in milliseconds). 
-
-    Values <10000 mean: Auto-save is off.
-  */
-  long int m_autoSaveInterval;
-
   void ShowTip(bool force);
 
   /*! Get the name of the help file
@@ -175,7 +170,7 @@ public:
   void SendMaxima(wxString s, bool history = false);
 
   //! Open a file
-  void OpenFile(wxString file,
+  bool OpenFile(wxString file,
                 wxString command = wxEmptyString);
 
   //! Does this document need saving?
@@ -187,6 +182,8 @@ public:
   { m_console->OpenHCaret(file, GC_TYPE_IMAGE); }
 
 private:
+  //! Is true if opening the file from the command line failed before updating the statusbar.
+  bool m_openInitialFileError;
   //! Escape strings into a format lisp accepts
   wxString EscapeForLisp(wxString str);
   //! The number of Jiffies Maxima had used the last time we asked
@@ -195,6 +192,8 @@ private:
   long long m_cpuTotalJiffies_old;
   //! Do we need to update the menus + toolbars?
   bool m_updateControls;
+  //! All configuration commands we still have to send to maxima
+  wxString m_configCommands;
   //! A RegEx that matches gnuplot errors.
   wxRegEx m_gnuplotErrorRegex;
   //! Clear the evaluation queue and return true if "Abort on Error" is set. 
@@ -211,8 +210,13 @@ private:
   int m_outputCellsFromCurrentCommand;
   //! The maximum number of lines per command we will display 
   int m_maxOutputCellsPerCommand;
-  //! The number of consecutive unsuccessful attempts to connect to the maxima server
-  int m_unsuccessfullConnectionAttempts;
+  /*! Double the number of consecutive unsuccessful attempts to connect to the maxima server
+
+    Each prompt is deemed as but one hint for a working maxima while each crash counts twice
+    which hinders us from endlessly restarting in case maxima crashes, outputs something 
+    seemingly sensible and crashes again.
+   */
+  int m_unsuccessfulConnectionAttempts;
   //! The current working directory maxima's file I/O is relative to.
   wxString m_CWD;
   //! Do we want to evaluate the file after startup?
@@ -227,12 +231,19 @@ private:
     If text doesn't contain any error this function returns wxEmptyString
   */
   wxString GetUnmatchedParenthesisState(wxString text,int &index);
-  //! The buffer all data from maxima is temporarily stored in.
-  unsigned char *m_packetFromMaxima;
   //! The buffer all text from maxima is stored in before converting it to a wxString.
   wxMemoryBuffer m_uncompletedChars;
+  /*! How many load() commands are currently nested?
+
+    We are only interested on the filenames of load() commands that are 
+    loaded by the user, but not the ones that are loaded by a loaded 
+    package.
+   */
+  int m_nestedLoadCommands;
 
 protected:
+  //! The gnuplot process info
+  wxProcess *m_gnuplotProcess;
   //! Is this window active?
   bool m_isActive;
   //! Called when this window is activated or deactivated.
@@ -263,6 +274,7 @@ protected:
   void CheckForUpdates(bool reportUpToDate = false);
 
   void OnRecentDocument(wxCommandEvent &event);
+  void OnRecentPackage (wxCommandEvent &event);
   void OnUnsavedDocument(wxCommandEvent &event);
 
   void OnChar(wxKeyEvent &event);
@@ -297,6 +309,7 @@ protected:
   void SimplifyMenu(wxCommandEvent &event);        //!< Processes "Simplify menu" clicks
   void PlotMenu(wxCommandEvent &event);            //!< Processes "Plot menu" cloicks
   void ListMenu(wxCommandEvent &event);            //!< Processes "list menu" clicks
+  void DrawMenu(wxCommandEvent &event);            //!< Processes "draw menu" clicks
   void NumericalMenu(wxCommandEvent &event);       //!< Processes "Numerical menu" clicks
   void HelpMenu(wxCommandEvent &event);            //!< Processes "Help menu" clicks
   void EditMenu(wxCommandEvent &event);            //!< Processes "Edit menu" clicks
@@ -310,6 +323,7 @@ protected:
    */
   void ShowPane(wxCommandEvent &event);            //<! Makes a sidebar visible
   void OnProcessEvent(wxProcessEvent &event);      //
+  void OnGnuplotClose(wxProcessEvent &event);      //
   void PopupMenu(wxCommandEvent &event);           //
   void StatsMenu(wxCommandEvent &event);           //
 
@@ -339,6 +353,14 @@ protected:
   //! Triggered when we get new chars from maxima.
   void OnNewChars();
 
+  /*! Add a parameter to an draw command
+
+    \param cmd The parameter to  add to the draw command
+    \param dimensionsOfNewDrawCommand The number of dimensions the new draw command needs to 
+                                      have if we need to create one..
+   */
+  void AddDrawParameter(wxString cmd, int dimensionsOfNewDrawCommand = 2);
+
   /* Append something to the console. Might be Text or XML maths.
 
     \return A pointer to the last line of Unicode text that was appended or 
@@ -385,6 +407,8 @@ protected:
 
   void TryUpdateInspector();
 
+  void UpdateDrawPane();
+  
   wxString ExtractFirstExpression(wxString entry);
 
   wxString GetDefaultEntry();
@@ -469,6 +493,13 @@ protected:
    */
   void ReadLoadSymbols(wxString &data);
 
+  //! Read (and discard) suppressed output
+  void ReadSuppressedOutput(wxString &data);
+
+    /*! Reads the variable values maxima advertises to us
+   */
+  void ReadVariables(wxString &data);
+
 #ifndef __WXMSW__
 
   //! reads the output the maxima command sends to stdout
@@ -518,7 +549,7 @@ protected:
    */
   void ResetTitle(bool saved, bool force = false);
 
-  void FirstOutput(wxString s);
+  void FirstOutput();
 
   /*! Opens a content.xml file that has been extracted from a broken .wxmx file
    */
@@ -560,6 +591,8 @@ protected:
   }
 
   wxSocketBase *m_client;
+  wxSocketInputStream *m_clientStream;
+  wxTextInputStream *m_clientTextStream;
   wxSocketServer *m_server;
   //! Is the network connection to maxima working?
   bool m_isConnected;
@@ -586,10 +619,18 @@ protected:
   wxString m_promptPrefix;
   //! The marker for the end of a input prompt
   wxString m_promptSuffix;
+  //! The marker for the start of a variables section
+  wxString m_variablesPrefix;
+  //! The marker for the end of a variables section
+  wxString m_variablesSuffix;
   //! The marker for the start of a list of autocompletion templates
   wxString m_symbolsPrefix;
   //! The marker for the end of a list of autocompletion templates
   wxString m_symbolsSuffix;
+  //! The marker that tells to start suppressing any output from maxima
+  wxString m_suppressOutputPrefix;
+  //! The marker that tells to stop to suppress any output from maxima
+  wxString m_suppressOutputSuffix;
   wxString m_firstPrompt;
   bool m_dispReadOut;               //!< what is displayed in statusbar
   bool m_inLispMode;                //!< don't add ; in lisp mode
@@ -598,12 +639,14 @@ protected:
   wxPrintData *m_printData;
   bool m_closing;
   wxString m_openFile;
+  wxString m_maximaTempDir;
   bool m_fileSaved;
-  bool m_variablesOK;
   wxString m_chmhelpFile;
   bool m_htmlHelpInitialized;
   wxString m_maximaVersion;
+  wxString m_maximaArch;
   wxString m_lispVersion;
+  wxString m_lispType;
   //! The Char the current command starts at in the current WorkingGroup
   int m_commandIndex;
 #if defined (__WXMSW__)
