@@ -19,6 +19,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
+//  SPDX-License-Identifier: GPL-2.0+
 
 /*!\file 
 This file declares the class wxMaximaFrame
@@ -40,7 +41,8 @@ surrounding the worksheet.
 #include <wx/notifmsg.h>
 
 #include "MathCtrl.h"
-#include "Setup.h"
+#include "RecentDocuments.h"
+#include "Version.h"
 #include "History.h"
 #include "ToolBar.h"
 #include "XmlInspector.h"
@@ -87,30 +89,28 @@ public:
       This event is assigned an ID higher than the highest ID wxWidgets assigns to 
       its internal events in order to avoid ID clashes.
     */
-            menu_pane_hideall = wxID_HIGHEST + 1,
+    menu_pane_hideall = wxID_HIGHEST + 2500,
     /*! Both used as the "toggle the math pane" command and as the ID of the math pane
 
       Since this enum is also used for iterating over the panes it is vital 
       that this entry stays that of the first pane in this enum.
     */
-            menu_pane_math,
-    menu_pane_history,    //!< Both the "toggle the history pane" command and the history pane
-    menu_pane_structure,        //!< Both the "toggle the structure pane" command and the structure
-    menu_pane_xmlInspector,        //!< Both the "toggle the xml monitor" command and the monitor pane
+    menu_pane_math,
+    menu_pane_history,      //!< Both the "toggle the history pane" command and the history pane
+    menu_pane_structure,    //!< Both the "toggle the structure pane" command and the structure
+    menu_pane_xmlInspector, //!< Both the "toggle the xml monitor" command and the monitor pane
     menu_pane_format,    //!< Both the "toggle the format pane" command and the format pane
-#ifdef wxUSE_UNICODE
-    menu_pane_greek,            //!< Both the "toggle the format pane" command for the "greek" pane
-    menu_pane_symbols,          //!< Both the "toggle the format pane" command for the "symbols" pane
-#endif
+    menu_pane_greek,     //!< Both the "toggle the format pane" command for the "greek" pane
+    menu_pane_log,       //!< Both the "toggle the format pane" command for the "debug" pane
+    menu_pane_draw,      //!< Both the "toggle the format pane" command for the "draw" pane
+    menu_pane_symbols,   //!< Both the "toggle the format pane" command for the "symbols" pane
     /*! Both used as the "toggle the stats pane" command and as the ID of the stats pane
 
       Since this enum is also used for iterating over the panes it is vital 
       that this entry stays that of the last pane in this enum.
     */
-            menu_pane_stats,
+    menu_pane_stats,
 
-    socket_client_id,
-    socket_server_id,
     input_line_id,
     refresh_id,
     menu_new_id,
@@ -167,6 +167,9 @@ public:
     menu_to_bfloat,
     menu_to_numer,
     menu_set_precision,
+    menu_set_displayprecision,
+    menu_engineeringFormat,
+    menu_engineeringFormatSetup,
     menu_functions,
     menu_variables,
     menu_clear_var,
@@ -207,6 +210,26 @@ public:
     menu_math_as_1D_ASCII,
     menu_math_as_2D_ASCII,
     menu_math_as_graphics,
+    menu_noAutosubscript,
+    menu_defaultAutosubscript,
+    menu_alwaysAutosubscript,
+    menu_roundedMatrixParens,
+    menu_roundedMatrixParensYes,
+    menu_roundedMatrixParensNo,
+    menu_draw_2d,
+    menu_draw_3d,
+    menu_draw_explicit,
+    menu_draw_implicit,
+    menu_draw_parametric,
+    menu_draw_points,
+    menu_draw_fgcolor,
+    menu_draw_fillcolor,
+    menu_draw_title,
+    menu_draw_key,
+    menu_draw_grid,
+    menu_draw_axis,
+    menu_draw_accuracy,
+    menu_draw_contour,
     button_factor_id,
     button_solve,
     button_solve_ode,
@@ -264,6 +287,8 @@ public:
     menu_copy_as_bitmap,
     menu_copy_as_svg,
     menu_save_as_svg,
+    menu_copy_as_emf,
+    menu_save_as_emf,
     menu_copy_as_rtf,
     menu_copy_to_file,
     menu_export_html,
@@ -325,6 +350,37 @@ public:
     menu_list_append_item,
     menu_list_append_list,
     menu_list_interleave,
+    menu_recent_packages,
+    menu_recent_package_0,
+    menu_recent_package_1,
+    menu_recent_package_2,
+    menu_recent_package_3,
+    menu_recent_package_4,
+    menu_recent_package_5,
+    menu_recent_package_6,
+    menu_recent_package_7,
+    menu_recent_package_8,
+    menu_recent_package_9,
+    menu_recent_package_10,
+    menu_recent_package_11,
+    menu_recent_package_12,
+    menu_recent_package_13,
+    menu_recent_package_14,
+    menu_recent_package_15,
+    menu_recent_package_16,
+    menu_recent_package_17,
+    menu_recent_package_18,
+    menu_recent_package_19,
+    menu_recent_package_20,
+    menu_recent_package_21,
+    menu_recent_package_22,
+    menu_recent_package_23,
+    menu_recent_package_24,
+    menu_recent_package_25,
+    menu_recent_package_26,
+    menu_recent_package_27,
+    menu_recent_package_28,
+    menu_recent_package_29,
     menu_recent_documents,
     menu_recent_document_0,
     menu_recent_document_1,
@@ -418,7 +474,11 @@ public:
     menu_edit_find,
     menu_history_previous,
     menu_history_next,
-    menu_check_updates
+    menu_check_updates,
+    socket_client_id,
+    socket_server_id,
+    maxima_process_id,
+    gnuplot_process_id
   };
 
   /*! Update the recent documents list
@@ -427,20 +487,7 @@ public:
     recent documents menu.
    */
   void UpdateRecentDocuments();
-
-  //! Add an entry to the "Recent Documents" list.
-  void AddRecentDocument(wxString file);
-
-  /*! Remove a file from the "Recent Documents" list.
-
-    Removing and re-adding a file will move it to the top of the list.
-   */
-  void RemoveRecentDocument(wxString file);
-
-  //! Read the nth entry in the list of recent documents.
-  wxString GetRecentDocument(int i)
-  { return m_recentDocuments[i]; }
-
+  
   /*! true, if a Pane is currently enabled
 
     \param id The event that toggles the visibility of the pane that is
@@ -571,8 +618,6 @@ protected:
   wxString GetTempAutosavefileName();
   //! Remember an temporary autosave file name.
   void RegisterAutoSaveFile();
-  //! Generates a list of all temporary autosave files we didn't open yet.
-  std::list<wxString> GetTempAutosaveFiles();
 private:
   //! A panel that shows all user-defined symbols on the symbols pane.
   wxPanel *m_userSymbols;
@@ -586,11 +631,8 @@ private:
 
   void do_layout();
 
-#if defined (__WXMSW__) || defined (__WXGTK20__) || defined (__WXMAC__)
-
   void SetupToolBar();
 
-#endif
 /*! 
   Create the menus.
 */
@@ -602,7 +644,42 @@ private:
 
   wxPanel *CreateFormatPane();
 
-  /*! A button for the greek pane
+  //! The class for the sidebar with the draw commands
+  class DrawPane: public wxPanel
+    {
+    public:
+      DrawPane(wxWindow *parent, int ID = wxID_ANY);
+      /*! Tell the sidebar if we currently are inside a 2D or a 3D plot command
+        
+        \param dimensions
+               - 0 = We aren't inside a plot
+               - 2 = We are inside a 2D plot
+               - 3 = We are inside a 3D plot
+      */
+      void SetDimensions(int dimensions);
+      int  GetDimensions() { return m_dimensions; }
+    private:
+      wxButton *m_draw_setup2d;
+      wxButton *m_draw_setup3d;
+      wxButton *m_draw_explicit;
+      wxButton *m_draw_implicit;
+      wxButton *m_draw_parametric;
+      wxButton *m_draw_points;
+      wxButton *m_draw_fgcolor;
+      wxButton *m_draw_fillcolor;
+      wxButton *m_draw_title;
+      wxButton *m_draw_key;
+      wxButton *m_draw_grid;
+      wxButton *m_draw_axis;
+      wxButton *m_draw_contour;
+      wxButton *m_draw_accuracy;
+      int m_dimensions;
+    };
+protected:
+  //! The sidebar with the draw commands
+  DrawPane *m_drawPane;
+private:
+  /*! A flat, compact button for the greek and the symbols pane
     
     \param parent The parent panel/window
     \param ch The unicode symbol
@@ -613,13 +690,12 @@ private:
    */
   wxPanel *CharButton(wxPanel *parent, wxChar ch, wxString description = wxEmptyString, bool matchesMaximaCommand = false);
 
-#ifdef wxUSE_UNICODE
+  wxPanel *CreateLogPane();
 
   wxPanel *CreateGreekPane();
 
   wxPanel *CreateSymbolsPane();
 
-#endif
 protected:
   //! The current length of the evaluation queue of commands we still need to send to maxima
   int m_EvaluationQueueLength;
@@ -637,9 +713,7 @@ protected:
 
   void CharacterButtonPressed(wxMouseEvent &event);
 
-  void LoadRecentDocuments();
-
-  void SaveRecentDocuments();
+  bool ToolbarIsShown();
 
   wxAuiManager m_manager;
   //! A XmlInspector-like xml monitor
@@ -650,8 +724,11 @@ protected:
   MathCtrl *m_console;
   //! The history pane
   History *m_history;
-  wxArrayString m_recentDocuments;
+  RecentDocuments m_recentDocuments;
+  RecentDocuments m_unsavedDocuments;
+  RecentDocuments m_recentPackages;
   wxMenu *m_recentDocumentsMenu;
+  wxMenu *m_recentPackagesMenu;
 };
 
 #endif // WXMAXIMAFRAME_H
